c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine aerody(ustar,elinv,z00,ra)
c
c-----ATDM v1.00
c
c     Estimates the aerodynamic resistance over solid
c     surfaces using the Weseley and Hicks (1977) and Sheih (1979)
c     relations for the similarity layer relations
c
c-----95/03/09:
c     -from UAM-V Revision 1.2  94/02/10  20:50:12  tcm
c     -modified ra from a 1-D array to a scalar
c
c     Input arguments:
c               ustar:          friction velocity (m/s)
c               elinv:          inverse Monin - Obukhov length (m)
c               z00:            surface roughness length (m)
c     Output arguments:
c               ra:             bulk aerodynamic resistance (s/m)
c     Called by:
c               surlyr    srlyrf
c
      data zr,vk/10., 0.40/
      data a1,a2,a3/0.598,0.390,0.090/
c
c-----estimate phi
c
      if(elinv.lt.0.0)then
        b3 = - a3*(alog(-zr*elinv)**2)
        b2 = a2*alog(-zr*elinv)
        phi = exp(a1 + b2 + b3)
      endif
      if(elinv.eq.0.0) phi = 0.0
      if(elinv.gt.0.0)then
        phi = -4.7*zr*elinv
      endif
c
c-----compute the aerodynamic resistance
c
      c1 = alog(zr/z00)
      c2 = 1.0/(vk*ustar)
c
c-----To prevent ra from going negative during unstable conditions
c
      if (phi.gt.0.99*c1) phi = 0.99*c1
c
      ra = c2*(c1 - phi)
      return
      end
