c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine bdrfil(ifine,nxf,nyf,nzf,pstrf)
c
c-----ATDM v1.00
c
c     Fills in boundary concentrations for a single fine
c     grid from the coarse grid concentrations surrounding
c     the fine grid
c
c-----95/06/09:
c     -from UAM-V Revision 1.2  94/02/10  20:50:36  tcm
c-----95/06/27 (GEM):
c     -modified calculation of BC's for ppm and sigma
c     -removed htt from argument list (uses sigmaf from common)
c      added pstrf to argument list (this simplifies pointer references
c      to the pstarf array which is also in common)
c
c     Input arguments:
c               ifine:          fine grid index   
c               nxf:            number of fine grid cells in x-direction
c               nyf:            number of fine grid cells in y-direction
c               nzf:            number of fine grid layers
c               pstrf:          array of fine grid P*'s
c     Called by:
c               atdm 
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'shrfin.cmd'
      include 'deffin.cmd'
      include 'comuam.cmd'
      include 'com1f.cmd'
      include 'ngmap.cmd'
c
      dimension pstrf(nxf,nyf)
c
c  fill in north boundary
c
      iedg = 1
      do 50 i=2,nxf-1
        if (igloc(i,iedg,ifine) .ne. 0) go to 50
        jc = jhloc(i,iedg,ifine)
        ic = int(fhloc(i,iedg,ifine)+.0005)+1
        do 40 n=1,nospec
          kget = 1
          do 30 k=1,nzf
            if (jend(ic) .le. jc) then
              bcf(i,iedg,n,k,ifine) = bc(ic,iedg,n,kget)*
     &                                       pstar(ic,jc)/pstrf(i,nyf)
            else
              bcf(i,iedg,n,k,ifine) = conc(ic,jc,n,kget)*
     &                                       pstar(ic,jc)/pstrf(i,nyf)
            endif
	    if (abs(sigmaf(k+1,ifine)-sigma(kget+1)).le.0.001)
     &        kget = kget + 1
   30     continue
c
   40   continue
   50 continue
c
c  fill in south boundary
c
      iedg = 2
      do 150 i=2,nxf-1
        if (igloc(i,iedg,ifine) .ne. 0) go to 150
        jc = jhloc(i,iedg,ifine)
        ic = int(fhloc(i,iedg,ifine)+.0005)+1
        do 140 n=1,nospec
          kget = 1
          do 130 k=1,nzf
            if (jstr(ic) .ge. jc) then
              bcf(i,iedg,n,k,ifine) = bc(ic,iedg,n,kget)*
     &                                       pstar(ic,jc)/pstrf(i,1)
            else
              bcf(i,iedg,n,k,ifine) = conc(ic,jc,n,kget)*
     &                                       pstar(ic,jc)/pstrf(i,1)
            endif
	    if (abs(sigmaf(k+1,ifine)-sigma(kget+1)).le.0.001)
     &        kget = kget + 1
  130     continue
c
  140   continue
  150 continue
c
c  fill in east boundary
c
      iedg = 3
      do 250 j=2,nyf-1
        if (igloc(j,iedg,ifine) .ne. 0) go to 250
        ic = jhloc(j,iedg,ifine)
        jc = int(fhloc(j,iedg,ifine)+.0005)+1
        do 240 n=1,nospec
          kget = 1
          do 230 k=1,nzf
            if (iend(jc) .le. ic) then
              bcf(j,iedg,n,k,ifine) = bc(jc,iedg,n,kget)*
     &                                       pstar(ic,jc)/pstrf(nxf,j)
            else
              bcf(j,iedg,n,k,ifine) = conc(ic,jc,n,kget)*
     &                                       pstar(ic,jc)/pstrf(nxf,j)
            endif
	    if (abs(sigmaf(k+1,ifine)-sigma(kget+1)).le.0.001)
     &        kget = kget + 1
  230     continue
c
  240   continue
  250 continue
c
c  fill in west boundary
c
      iedg = 4
      do 350 j=2,nyf-1
        if (igloc(j,iedg,ifine) .ne. 0) go to 350
        ic = jhloc(j,iedg,ifine)
        jc = int(fhloc(j,iedg,ifine)+.0005)+1
        do 340 n=1,nospec
          kget = 1
          do 330 k=1,nzf
            if (istr(jc) .ge. ic) then
              bcf(j,iedg,n,k,ifine) = bc(jc,iedg,n,kget)*
     &                                       pstar(ic,jc)/pstrf(1,j)
            else
              bcf(j,iedg,n,k,ifine) = conc(ic,jc,n,kget)*
     &                                       pstar(ic,jc)/pstrf(1,j)
            endif
	    if (abs(sigmaf(k+1,ifine)-sigma(kget+1)).le.0.001)
     &        kget = kget + 1
  330     continue
c
  340   continue
  350 continue
c
      return
      end
