c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine diagn(time,idate,m,tend,tstr,idystr
     &,idaynf)
c
c-----ATDM v1.00
c
c     Sets allowable data intervals, calculates min, max,
c     and average data values, and checks these against allowable values
c
c-----95/02/22:
c     -from UAM-V Revision 1.3  94/02/17  09:07:31  tcm
c     -removed variables and code performing bounds checking on old 
c      linear chemistry inputs
c     -modified code to check PTSRCX & PTSCRY variables (see hedpts)
c     -removed code that initialized unused AHPREC array
c-----95/08/24:
c     -Modified to calculate min/max/mean over active columns only (i.e.,
c      non-boundary cells) and for better output formats
c-----95/10/04:
c     -Revised checks on stack parameters to indicate when vexit > maxv,
c      and to write out more clear message for other stack problems
c
c     Input arguments:
c               time:           model time    
c               idate:          model date
c               m:              flag to check data from setup (1) or update (2)
c               tend:           model end time
c               tstr:           model start time 
c               idystr:         model start date
c               idaynf:         model end date
c     Files written:
c               8:              diagnostic file
c     Routines called:
c               chkdat
c     Called by:
c               setup     update
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'comuam.cmd'
      include 'com3.cmd'
      include 'comchem.cmd'
      include 'latlon.cmd'
      include 'stack.cmd'
      include 'ctlflg.cmd'
c
c      real avmaxj(mxx,mxz),aumaxi(mxy,mxz)
      real mindtd,maxdtd,mindtp,maxdtp,mindx,maxdx,mindy,maxdy
      real minsth,maxsth
      real minstd,maxstd,mintst,maxtst,minv,maxv
      real khmax(mxz),khmin(mxz),khavg(mxz)
      real kvmax(mxz),kvmin(mxz),kvavg(mxz)
      logical errflg
      dimension cnmin(mxspec,mxz),cnmax(mxspec,mxz),cnsum(mxspec,mxz)
      dimension umin(mxz),umax(mxz),uavg(mxz),vmin(mxz),vmax(mxz)
      dimension vavg(mxz)
      dimension awsmin(mxz),awsmax(mxz),awsavg(mxz)
      dimension htmin(mxz),htmax(mxz),htavg(mxz)
      dimension psmin(mxz),psmax(mxz),psavg(mxz)
      dimension cldmin(mxz),cldmax(mxz),cldavg(mxz)
      dimension vdpmin(mxspec),vdpmax(mxspec),vdpavg(mxspec)
      dimension tmpmin(mxz),tmpmax(mxz),tmpav(mxz)
      dimension watmin(mxz),watmax(mxz),watav(mxz)
      dimension o3min(mxz),o3max(mxz),o3av(mxz)
      data errflg /.false./
      data mindtd,maxdtd /30.,600./
      data mindtp,maxdtp /30.,600./
      data mindx,mindy,maxdx,maxdy /0.01,0.01,20.,20./
      data minnox,maxnox /5,mxx/
      data minnoy,maxnoy /5,mxy/
      data tzmin,tzmax /0.,9./
      data maxnpt /mxsrc/
      data minsth,maxsth /0.5,2100./
      data minstd,maxstd /0.01,100./
      data mintst,maxtst /260.,2000./
      data minv,maxv /0.0001,500./
c
c *** do we check setup data or update data
c
      if(m.gt.1) go to 1000
c
c *** This portion is executed at setup time only
c
      if(idystr.gt.idaynf) go to 9001
      if((idystr.eq.idaynf).and.(tstr.gt.tend)) go to 9001
  101 call chkdat(idystr,errflg)
      call chkdat(idaynf,errflg)
      if((tstr.lt.0.).or.(tstr.gt.2400.)) go to 9002
  102 if((tend.lt.0.).or.(tend.gt.2400.)) go to 9003
  103 if(dtdata.lt.mindtd) go to 9005
      if(dtdata.gt.maxdtd) go to 9005
  105 if(dtprnt.lt.mindtp) go to 9007
      if(dtprnt.gt.maxdtp) go to 9007
  107 if(dlong.lt.mindx) go to 9009
      if ((.not.lcart).and.(dlong.gt.maxdx)) go to 9009
      if (lcart.and.dlong.gt.200.0) go to 9009
  109 if (.not.lcart.and.(dlat.gt.maxdy)) go to 9011
      if(dlat.lt.mindy) go to 9011
      if(lcart.and.dlat.gt.200.0) go to 9011
  111 if(nox.lt.minnox) go to 9013
      if(nox.gt.maxnox) go to 9013
  113 if(noy.lt.minnoy) go to 9015
      if(noy.gt.maxnoy) go to 9015
  115 if(nospec.gt.mxspec) go to 9016
      if (nospec.lt.1) go to 9016
  116 continue
  118 if((wlong.lt.(-180.)).or.(wlong.gt.180.)) go to 9019
  119 if((elong.lt.(-180.)).or.(elong.gt.180.)) go to 9020
  120 if((nlat.lt.(-90.)).or.(nlat.gt.90.)) go to 9021
  121 if((slat.lt.(-90.)).or.(slat.gt.90.)) go to 9022
  122 if((tz.lt.tzmin).or.(tz.gt.tzmax)) go to 9023
  123 if(lpts .and. nopts.lt.1) go to 9031
      if(nopts.gt.maxnpt) go to 9031
  131 do 20 i=1,nopts
        if(ptsrcx(i).gt.float(nox)) goto 9037
        if(ptsrcx(i).lt.0.0) goto 9037
  137   if(ptsrcy(i).gt.float(noy)) goto 9038
        if(ptsrcy(i).lt.0.0) goto 9038
  138   continue
  139   if((stkht(i).lt.minsth).or.(stkht(i).gt.maxsth)) go to 9040
  140   if((stkd(i).lt.minstd).or.(stkd(i).gt.maxstd)) go to 9041
  141   if((temps(i).lt.mintst).or.(temps(i).gt.maxtst)) go to 9042
  142   if((vexit(i).lt.minv).or.(vexit(i).gt.maxv)) go to 9043
  143   if (vexit(i).gt.maxv) vexit(i)=maxv
        go to 20
 9037   write(8,8037) i,ptsrcx(i)
        go to 137
 9038   write(8,8038) i,ptsrcy(i)
        go to 138
 9040   write(8,8040) i,stkht(i)
        go to 140
 9041   write(8,8041) i,stkd(i)
        go to 141
 9042   write(8,8042) i,temps(i)
        go to 142
 9043   write(8,8043) i,vexit(i)
        go to 143
   20 continue
c
      do 35 l=1,nlayer
        do 34 k=1,nospec
          ncell = 0
          do 30 j=1,noy
            do 29 i=1,nox
              if (i.le.istr(j) .or. i.ge.iend(j)) goto 29
                ncell = ncell + 1
                if (ncell.eq.1) then
                  cnmin(k,l)=conc(i,j,k,l)
                  cnmax(k,l)=conc(i,j,k,l)
                  cnsum(k,l)=conc(i,j,k,l)
                else
                  cnmax(k,l)=amax1(cnmax(k,l),conc(i,j,k,l))
                  cnmin(k,l)=amin1(cnmin(k,l),conc(i,j,k,l))
                  cnsum(k,l)=cnsum(k,l)+conc(i,j,k,l)
                endif
   29       continue
   30     continue
          cnsum(k,l)=cnsum(k,l)/ncell
   34   continue
   35 continue
c
      write(8,901)
  901 format(/,20x,'Initial Concentrations')
      write(8,'(a,i5)')'Number of active cells: ',ncell
      write(8,903)
  903 format(//,27x,'Minimum',5x,'Maximum',5x,'Average')
      do 990 k=1,nospec
      kk=k
      do 990 l=1,nlayer
      write(8,902) l,(mspec(mm,k),mm=1,10),cnmin(k,l),
     &    cnmax(k,l),cnsum(k,l)
  990 continue
  902 format(1x,'layer ',i2,1x,10a1,4x,1pe10.2,2x,e10.2,2x,e10.2)
      if(errflg) go to 10000
      write(8,41)
   41 format(5x,'All variables read in setup lie within',
     &   ' allowable limit')
      write (*,*) 'end of diagn-1'
      return
10000 write(8,42)
   42 format(/,5x,'Run aborted because of preceding error',
     &   ' message(s)')
      stop 1
c
c *** check update data
c
 1000 continue
c
c *** compute min,max,av of input values
c
      ncell = 0
      nrcell = 0
      do 51 j=1,noy
        do 50 i=1,nox
          if (i.le.istr(j) .or. i.ge.iend(j)) goto 50
          ncell = ncell + 1
          do 48 k=1,nlayer
            if (ncell.eq.1) then
              umin(k)=u(i,j,k)
              umax(k)=umin(k)
              uavg(k)=umin(k)
              aumin(k)=abs(u(i,j,k))
              aumax(k)=aumin(k)
	      aumaxi(j,k)=aumax(k)
              auavg(k)=aumin(k)
              vmin(k)=v(i,j,k)
              vmax(k)=vmin(k)
              vavg(k)=vmin(k)
              avmin(k)=abs(v(i,j,k))
              avmax(k)=avmin(k)
              avmaxj(i,k)=avmin(k)
              avavg(k)=avmin(k)
              awsmin(k)=sqrt(u(i,j,k)*u(i,j,k)+v(i,j,k)*v(i,j,k))
              awsmax(k)=awsmin(k)
              awsavg(k)=awsmin(k)
              tmpmin(k)=temper(i,j,k)
              tmpmax(k)=tmpmin(k)
              tmpav(k)=tmpmin(k)
              watmin(k)=ch2o(i,j,k)
              watmax(k)=watmin(k)
              watav(k)=watmin(k)
              khmin(k)=kh(i,j,k)
              khmax(k)=khmin(k)
              khavg(k)=khmin(k)
              kvmax(k)=kv(i,j,k)
              kvmin(k)=kvmax(k)
              kvavg(k)=kvmax(k)
              htmin(k)=h(i,j,k)
              htmax(k)=htmin(k)
              htavg(k)=htmin(k)
              psmin(k)=press(i,j,k)
              psmax(k)=psmin(k)
              psavg(k)=psmin(k)
              cldmax(k)=cloud(i,j,k)
              cldmin(k)=cldmax(k)
              cldavg(k)=cldmax(k)
              o3max(k) = ozone(i,j,k)
              o3min(k) = o3max(k)
              o3av(k) = o3max(k)
            else
            umin(k)=amin1(umin(k),u(i,j,k))
            umax(k)=amax1(umax(k),u(i,j,k))
            uavg(k)=uavg(k)+u(i,j,k)
            aumin(k)=amin1(aumin(k),abs(u(i,j,k)))
            aumax(k)=amax1(aumax(k),abs(u(i,j,k)))
	    aumaxi(j,k)=amax1(aumaxi(j,k),abs(u(i,j,k)))
            auavg(k)=auavg(k)+abs(u(i,j,k))
            vmin(k)=amin1(vmin(k),v(i,j,k))
            vmax(k)=amax1(vmax(k),v(i,j,k))
            vavg(k)=vavg(k)+v(i,j,k)
            avmin(k)=amin1(avmin(k),abs(v(i,j,k)))
            avmax(k)=amax1(avmax(k),abs(v(i,j,k)))
            avmaxj(i,k)=amax1(avmaxj(i,k),abs(v(i,j,k)))
            avavg(k)=avavg(k)+abs(v(i,j,k))
            wspeed=sqrt(u(i,j,k)*u(i,j,k)+v(i,j,k)*v(i,j,k))
            awsmin(k)=amin1(awsmin(k),wspeed)
            awsmax(k)=amax1(awsmax(k),wspeed)
            awsavg(k)=awsavg(k)+wspeed
            khmin(k)=amin1(khmin(k),kh(i,j,k))
            khmax(k)=amax1(khmax(k),kh(i,j,k))
            khavg(k)=khavg(k)+kh(i,j,k)
            kvmin(k)=amin1(kvmin(k),kv(i,j,k))
            kvmax(k)=amax1(kvmax(k),kv(i,j,k))
            kvavg(k)=kvavg(k)+kv(i,j,k)
            htmin(k)=amin1(htmin(k),h(i,j,k))
            htmax(k)=amax1(htmax(k),h(i,j,k))
            htavg(k)=htavg(k)+h(i,j,k)
            psmin(k)=amin1(psmin(k),press(i,j,k))
            psmax(k)=amax1(psmax(k),press(i,j,k))
            psavg(k)=psavg(k)+press(i,j,k)
            tmpmin(k)=amin1(tmpmin(k),temper(i,j,k))
            tmpmax(k)=amax1(tmpmax(k),temper(i,j,k))
            tmpav(k)=tmpav(k)+temper(i,j,k)
            watmin(k)=amin1(watmin(k),ch2o(i,j,k))
            watmax(k)=amax1(watmax(k),ch2o(i,j,k))
            watav(k)=watav(k)+ch2o(i,j,k)
            cldmax(k)=amax1(cldmax(k),cloud(i,j,k))
            cldmin(k)=amin1(cldmin(k),cloud(i,j,k))
            cldavg(k)=cldavg(k)+cloud(i,j,k)
            o3max(k)=amax1(o3max(k),ozone(i,j,k))
            o3min(k)=amin1(o3min(k),ozone(i,j,k))
            o3av(k)=o3av(k)+ozone(i,j,k)
            endif
   48     continue
          do 49 l=1,nospec
            if (ncell.eq.1) then
              vdpmin(l)=depvel(i,j,l)
              vdpmax(l)=vdpmin(l)
              vdpavg(l)=vdpmin(l)
            else 
              vdpmin(l)=amin1(vdpmin(l),depvel(i,j,l))
              vdpmax(l)=amax1(vdpmax(l),depvel(i,j,l))
              vdpavg(l)=vdpavg(l)+depvel(i,j,l)
            endif
   49     continue
          if (rain(i,j).ne.0.) then
            nrcell = nrcell + 1
            if (nrcell.eq.1) then
              rnmax=rain(i,j)
              rnmin=rain(i,j)
              rnsum=rain(i,j)
            else
              rnmax=amax1(rnmax,rain(i,j))
              rnmin=amin1(rnmin,rain(i,j))
              rnsum=rnsum+rain(i,j)
            endif
          endif
   50   continue
   51 continue
c
      do 52 k=1,nlayer
        uavg(k)=uavg(k)/ncell
        vavg(k)=vavg(k)/ncell
        auavg(k)=auavg(k)/ncell
        avavg(k)=avavg(k)/ncell
        awsavg(k)=awsavg(k)/ncell
        tmpav(k)=tmpav(k)/ncell
        watav(k)=watav(k)/ncell
        khavg(k)=khavg(k)/ncell
        kvavg(k)=kvavg(k)/ncell
        htavg(k)=htavg(k)/ncell
        psavg(k)=psavg(k)/ncell
        cldavg(k)=cldavg(k)/ncell
        o3av(k)=o3av(k)/ncell
   52 continue
      do 521 l=1,nospec
        vdpavg(l)=vdpavg(l)/ncell
  521 continue
      if ( nrcell .gt. 0 ) rnav=rnsum/nrcell
      do 54 k=1,nospec
        nbc = 0
        do 53 l=1,nlayer
          cnmin(k,l)=1.e20
          cnmax(k,l)=0.0
          cnsum(k,l)=0.0
   53   continue
        do 57 l=1,nlayer
          do 55 i=1,nox
            if (bc(i,1,k,l) .ge. 0.) then
              cnmin(k,l)=amin1(cnmin(k,l),bc(i,1,k,l))
              cnmax(k,l)=amax1(cnmax(k,l),bc(i,1,k,l))
              cnsum(k,l)=cnsum(k,l)+bc(i,1,k,l)
              if (l.eq.1) nbc = nbc + 1
            endif
            if (bc(i,2,k,l) .ge. 0.) then
              cnmin(k,l)=amin1(cnmin(k,l),bc(i,2,k,l))
              cnmax(k,l)=amax1(cnmax(k,l),bc(i,2,k,l))
              cnsum(k,l)=cnsum(k,l)+bc(i,2,k,l)
              if (l.eq.1) nbc = nbc + 1
            endif
   55     continue
   57   continue
        do 58 l=1,nlayer
          do 56 i=1,noy
            if (bc(i,3,k,l) .ge. 0.) then
              cnmin(k,l)=amin1(cnmin(k,l),bc(i,3,k,l))
              cnmax(k,l)=amax1(cnmax(k,l),bc(i,3,k,l))
              cnsum(k,l)=cnsum(k,l)+bc(i,3,k,l)
              if (l.eq.1) nbc = nbc + 1
            endif
            if (bc(i,4,k,l) .ge. 0.) then
              cnmin(k,l)=amin1(cnmin(k,l),bc(i,4,k,l))
              cnmax(k,l)=amax1(cnmax(k,l),bc(i,4,k,l))
              cnsum(k,l)=cnsum(k,l)+bc(i,4,k,l)
              if (l.eq.1) nbc = nbc + 1
            endif
   56     continue
   58   continue
      do 54 l=1,nlayer
        cnsum(k,l)=cnsum(k,l)/nbc
   54 continue
      write(8,5) time,idate
    5 format(/,'Time ',f5.0,'   Date ',i6,/,
     &       'Updated input variables have the following ranges')
      write(8,'(a,i5)')'Number of active cells: ',ncell
      write(8,6)
    6 format(//,27x,'Minimum',5x,'Maximum',5x,'Average',/)
c
      do 70 l=1,nlayer
        write(8,7001) l,16.67*umin(l),16.67*umax(l),16.67*uavg(l)
        write(8,7002) l,16.67*vmin(l),16.67*vmax(l),16.67*vavg(l)
   70 continue
c
      do 71 l=1,nlayer
        write(8,7003) l,16.67*aumin(l),16.67*aumax(l),16.67*auavg(l)
        write(8,7004) l,16.67*avmin(l),16.67*avmax(l),16.67*avavg(l)
   71 continue
c
      do 72 l=1,nlayer
        write(8,7005) l,16.67*awsmin(l),16.67*awsmax(l),16.67*awsavg(l)
   72 continue
c
      do 733 l=1,nlayer
        write(8,7088) l,psmin(l),psmax(l),psavg(l)
  733 continue
c
      do 73 l=1,nlayer
        write(8,7008) l,htmin(l),htmax(l),htavg(l)
   73 continue
c 
      do 74 l=1,nlayer
        write(8,7009) l,khmin(l),khmax(l),khavg(l)
   74 continue
c
      do 774 l=1,nlayer
        write(8,7079) l,kvmin(l),kvmax(l),kvavg(l)
  774 continue
c
      do 75 l=1,nlayer
        write(8,7033) l,tmpmin(l),tmpmax(l),tmpav(l)
   75 continue
c
      do 76 l=1,nlayer
        write(8,7034) l,watmin(l),watmax(l),watav(l)
   76 continue
c
      if (lpmchm .or. ltxchm) then
        do 761 l=1,nlayer
          write(8,7035)l,cldmin(l),cldmax(l),cldavg(l)
  761   continue
c
        do 762 l=1,nlayer
          write(8,7036)l,o3min(l),o3max(l),o3av(l)
  762   continue
      endif
c
      if (lwet) write(8,7078) rnmin,rnmax,rnav
c
      do 60 k=1,nospec
        kk=k
        write(8,7021)(mspec(mm,kk),mm=1,10),vdpmin(k),vdpmax(k),
     &     vdpavg(k)
        write(8,'(a,i4)') 'Number of boundary cells: ',nbc 
        do 77 l=1,nlayer
          write(8,7020) l,(mspec(mm,kk),mm=1,10),cnmin(k,l),
     &     cnmax(k,l),cnsum(k,l)
   77   continue
   60 continue
c
 7001 format(1x,'layer ',i2,' u (m/s)',7x,1pe10.2,2x,e10.2,2x,e10.2)
 7002 format(1x,'layer ',i2,' v (m/s)',7x,1pe10.2,2x,e10.2,2x,e10.2)
 7003 format(1x,'layer ',i2,' abs(u)',8x,1pe10.2,2x,e10.2,2x,e10.2)
 7004 format(1x,'layer ',i2,' abs(v)',8x,1pe10.2,2x,e10.2,2x,e10.2)
 7005 format(1x,'layer ',i2,' tot speed',5x,1pe10.2,2x,e10.2,2x,e10.2)
 7008 format(1x,'layer ',i2,' z (m)',9x,1pe10.2,2x,e10.2,2x,e10.2)
 7088 format(1x,'layer ',i2,' p (mb)',8x,1pe10.2,2x,e10.2,2x,e10.2)
 7078 format(1x,'rain fall (in/hr)',6x,e10.2,2x,1pe10.2,2x,e10.2)
 7009 format(1x,'layer ',i2,' kh (km2/min)',2x,1pe10.2,2x,e10.2,
     &       2x,e10.2)
 7079 format(1x,'layer ',i2,' kv (m2/s)',5x,1pe10.2,2x,e10.2,2x,e10.2)
 7033 format(1x,'layer ',i2,' temp (k)',6x,1pe10.2,2x,e10.2,2x,e10.2)
 7034 format(1x,'layer ',i2,' h2o (g/g)',5x,1pe10.2,2x,e10.2,2x,e10.2)
 7035 format(1x,'layer ',i2,' cloud cover',3x,1pe10.2,2x,e10.2,2x,e10.2)
 7036 format(1x,'layer ',i2,' ozone (ppm)',3x,1pe10.2,2x,e10.2,2x,e10.2)
 7020 format(1x,'layer ',i2,' bc ',10a1,1x,1pe10.2,2x,e10.2,2x,e10.2)
 7021 format(1x,'dep vel (m/s)',10a1,1pe10.2,2x,e10.2,2x,e10.2)
c
      write (*,*) 'end of diagn-2'
      return
c
 9001 write(8,8001) idystr,tstr,idaynf,tend
      errflg=.true.
      write(8,8001)
      go to 101
 9002 errflg=.true.
      write(8,8002) tstr
      go to 102
 9003 errflg=.true.
      write(8,8003) tend
      go to 103
 9005 errflg=.true.
      write(8,8005) dtdata
      go to 105
 9007 errflg=.true.
      write(8,8007) dtprnt
      go to 107
 9009 errflg=.true.
      write(8,8009) deltax
      go to 109
 9011 errflg=.true.
      write(8,8011) deltay
      go to 111
 9013 errflg=.true.
      write(8,8013) nox
      go to 113
 9015 errflg=.true.
      write(8,8015) noy
      go to 115
 9016 errflg=.true.
      write(8,8016) nospec
      go to 116
 9019 errflg=.true.
      write(8,8019) wlong
      go to 119
 9020 errflg=.true.
      write(8,8020) elong
      go to 120
 9021 errflg=.true.
      write(8,8021) nlat
      go to 121
 9022 errflg=.true.
      write(8,8022) slat
      go to 122
 9023 errflg=.true.
      write(8,8023) tz
      go to 123
 9031 errflg=.true.
      write(8,8031) nopts
      go to 131
 8001 format(5x,'starting date=',i10,10x,'starting time=',f10.2,
     &   /,5x,'Ending date=',i10,10x,'Ending time=',f10.2,
     &   //'Ending time precedes starting time. Please correct',
     &   'input data')
 8002 format(5x,'tstr=',e10.4,20x,'This is not a permissible value')
 8003 format(5x,'tend=',e10.4,20x,'This is not a permissible value')
 8005 format(5x,'dtdata=',e10.4,20x,'This is not a permissible value')
 8007 format(5x,'dtprnt=',e10.4,20x,'This is not a permissible value')
 8009 format(5x,'deltax=',e10.4,20x,'This is not a permissible value')
 8011 format(5x,'deltay=',e10.4,20x,'This is not a permissible value')
 8013 format(5x,'nox=',i10,20x,'A domain of 5 x 5 grid points is a ',
     &       'minimum requirement.')
 8015 format(5x,'noy=',i10,20x,'A domain of 5 x 5 grid points is a ',
     &       'minimum requirement.')
 8016 format(5x,'nospec=',i10,20x,'This is not a permissible value')
 8019 format(5x,'wlong=',e10.4,20x,'This is not a permissible value')
 8020 format(5x,'elong=',e10.4,20x,'This is not a permissible value')
 8021 format(5x,'nlat=',e10.4,20x,'This is not a permissible value')
 8022 format(5x,'slat=',e10.4,20x,'This is not a permissible value')
 8023 format(5x,'tz=',e10.4,20x,'This is not a permissible value')
 8031 format(5x,'nopts=',i10,20x,'This is not a permissible value')
 8037 format(5x,'WARNING: At source ',i5,' xpts=',e10.4,
     & ' This is outside the modeling region.',/
     & ' Emissions from this source will not be included in the run.')
 8038 format(5x,'WARNING: at source ',i5,' ypts=',e10.4,
     & ' This is outside the modeling region.',/
     & ' Emissions from this source will not be included in the run.')
 8039 format(5x,'WARNING: at source ',i5,' tsum=',e10.4,
     & ' This is outside the internal check range, but will be used')
 8040 format(5x,'WARNING: at source ',i5,' stkht=',e10.4,
     & ' This is outside the internal check range, but will be used')
 8041 format(5x,'WARNING: at source ',i5,' stkd=',e10.4,
     & ' This is outside the internal check range, but will be used')
 8042 format(5x,'WARNING: at source ',i5,' temps=',e10.4,
     & ' This is outside the internal check range, but will be used')
 8043 format(5x,'ERROR: At source ',i5,' vexit=',e10.4,
     & ' This is not a permissible value; reset to max: ',e10.4)
      end
