c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine diffus
c
c-----ATDM v1.00
c
c     Calculates horizontal diffusion coefficients for coarse grid based
c     on scale considerations and deformation of the windfield
c
c-----95/05/30:
c     -from UAM-V Revision 1.2  94/02/10  20:54:47  tcm
c     -hardwired difcof, difmin, difmax here
c     -altered deltay**2 to deltay*deltax(j)
c-----95/08/24:
c     -Altered loop over (i,j) for just active cells, rather than full 
c      (1 to nox,1 to noy)
c
c     Default value for DIFCOF is 0.25/SQRT(2) 
c
c     Some comments regarding optional staggered u/v grids:
c     For grids with u & v at cell centers, gradients wrt x are calculated
c     at i + 1/2 and gradients wrt y are calculated at j + 1/2.
c     KH is used as if at i + 1/2 in STEPX, but as if at
c     j + 1/2 in STEPY.  On the other hand, in the case of
c     staggered u & v grids, dudx and dvdy are calculated at
c     cell centers and dudy and dvdx are calculated at
c     (i + 1/2, j + 1/2).  KH will still be used as if at
c     i + 1/2 in STEPX and j + 1/2 in STEPY.  In the staggered
c     grid, there is not sufficient information to calculate
c     all the gradients for i = 1 or j = 1.  Rather than 
c     calculate DEF based on incomplete data, DEF is set
c     to zero for these cases.  So that all boundaries are 
c     treated equivalently in either the staggered or cell 
c     centered case, DEF is set to zero for i = 1 or nx
c     or j = 1 or ny.
c     (tcm, 7/15/91)
c
c     Called by:
c               update 
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'comuam.cmd'
      include 'com3.cmd'
      include 'advec.cmd'
c
      difcof=0.1767767
      difmax=6.0
      difmin=0.0006
c
      do 150 k=1,nlayer
        do 100 j=1,noy
          do 90 i=1,nox
            if (i .le. istr(j) .or.
     &          j .le. jstr(i) .or.
     &          i .ge. iend(j) .or.
     &          j .ge. jend(i)) then
              def = 0.
            else
c
c-----Staggered winds
c
              if (lstagr) then
                u2=u(i,j,k)
                u1=u(i-1,j,k)
                v2=v(i+1,j,k)
                v1=v(i,j,k)
                dudx= (u2-u1)/deltax(j)
                dvdx= (v2-v1)/deltax(j)
c
                u2=u(i,j+1,k)
                u1=u(i,j,k)
                v2=v(i,j,k)
                v1=v(i,j-1,k)
                dudy=(u2-u1)/deltay
                dvdy=(v2-v1)/deltay
              else
c
c-----Cell-centered winds
c
                u2=u(i+1,j,k)
                u1=u(i,j,k)
                v2=v(i+1,j,k)
                v1=v(i,j,k)
                dudx= (u2-u1)/deltax(j)
                dvdx= (v2-v1)/deltax(j)
c
                u2=u(i,j+1,k)
                u1=u(i,j,k)
                v2=v(i,j+1,k)
                v1=v(i,j,k)
                dudy=(u2-u1)/deltay
                dvdy=(v2-v1)/deltay
              endif
              def=sqrt((dudy+dvdx)**2 + (dudx-dvdy)**2)
            endif
c
c-----Calculate Kh
c
            kh(i,j,k)=difcof*(deltay*deltax(j))*def
            if (kh(i,j,k).gt.difmax) kh(i,j,k)=difmax
            if (kh(i,j,k).lt.difmin) kh(i,j,k)=difmin
c
   90     continue
  100   continue
  150 continue
      return
      end
