c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine dtsize(time,ndate,deltat)
c
c-----ATDM v1.00
c
c     Determines time step size for coarse grid
c
c-----95/06/07:
c     -from UAM-V Revision 1.2  94/02/10  20:54:55  tcm
c
c     Input arguments:
c               time:           model time   
c               ndate:          model date
c     Output arguments:
c               deltat:         coarse grid time step (min)
c     Files written:
c               6:              standard output file unit (*)
c     Called by:
c               atdm 
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'com3.cmd'
      include 'dtksub.cmd'
      real dt1k(mxz),diff(mxz),dt2k(mxz),dtvalk(mxz)
      real dtik(mxy,mxz),dtjk(mxx,mxz)
      data dt1k /mxz*1.e20/
      data diff /mxz*0.0/
c
c-----initialize some contants
c
c       dtsub(1)=10.
c       dtsub(2)=10.
c       dtsub(3)=10.
c       deltat=10.
       do k=1,mxz
	 do i=1,mxx
	   dtjk(i,k)=1.e20
	 enddo
	 do j=1,mxy
           dtik(j,k)=1.e20
	 enddo
       enddo
c
       deltat=1.e20
       dt1=1.e20
       dtmax=1.e-20
       dtmin=1.e20
c
      do 100 j=1,noy
c
c     check if out of modeling region
c
        if (istr(j).gt.nox) go to 100
c
        is=istr(j)
        ie=iend(j)
c
c       find the minimum horizontal grid resolution to calculate
c       time step later
c
        if(deltax(j) .gt. deltay)then
          horzds=deltay
        else
          horzds=deltax(j)
        endif
c
c       calculate by each column element
c
        do 90 i=is,ie
          if (j.lt.jstr(i)) go to 90
          if (j.gt.jend(i)) go to 90
          do 80 k=1,nlayer
            diff(k)=amax1(diff(k),kh(i,j,k)/horzds**2)
	    dtik(j,k)=amin1(dtik(j,k),0.48*deltax(j)/aumaxi(j,k))
	    dtjk(i,k)=amin1(dtjk(i,k),0.48*deltay/avmaxj(i,k))
   80     continue
   90   continue
c
  100 continue
c
c      do 91 j=1,noy
c	write(*,*) 'j, dtik :',j,(dtik(j,k),k=1,nlayer)
c 91   continue
c      do 92 i=1,nox
c	write(*,*) 'i, dtjk :',i,(dtjk(i,k),k=1,nlayer)
c 92   continue
c
      dtval=amin1(dtdata*0.6,dtprnt*0.6)
c
      do 130 k=1,nlayer

        if(diff(k).gt.0.)then
          dt2k(k)=0.15/diff(k)
        else
          dt2k(k)=1.e20
        endif
c
c     determine maximum deltat for this time step
c
c        deltatk(k)=amin1(dt1k(k),dt2k(k),dtstep)
        deltatk(k)=amin1(dt2k(k),dtstep)
	deltat=amin1(deltat,deltatk(k))
	dtmax=amax1(dtmax,deltatk(k))
	dtmin=amin1(dtmin,deltatk(k))
c
 130  continue
c
c     make time step an even multiple of print or update time
c
c97/01/15      deltat=dtstep
      deltat=dtmax
      dtsub(1)=deltat
      dtsub(2)=deltat
      nstps(1)=int(deltat/dtsub(1))
      rnchk=amod(dtsub(1),dtmax)
      if ( rnchk .ge. 0.001 ) then
	 nstp=int(dtsub(1)/dtmax)+1
	 dtmax=dtsub(1)/float(nstp)
      endif
c      write(*,*) ' dtmax,deltatk :',dtmax,(deltatk(k),k=1,nlayer)
c
      do 141 k=1,nlayer
	 if ( deltatk(k) .ge. dtmax ) deltatk(k)=dtmax
 	 nstepk(k)=int(dtmax/deltatk(k))
         rnchk=amod(deltatk(k),dtmax)
	 if ( rnchk .ge. 0.001 ) then
	    nstepk(k)=nstepk(k)+1
	    deltatk(k)=dtmax/float(nstepk(k))
	 endif
c	 write(*,*) ' k,dt,nstep :',k,deltatk(k),nstepk(k)
 141  continue

      dtsub(3)=dtmax
c**************************
c      dtsub(1)=dtmax/2.
c      dtsub(2)=dtmax/2.
c      dtsub(3)=dtmax/2.
c      deltat=dtsub(1)
c**************************
      do n=2,3
	 nstps(n)=int(dtsub(n-1)/dtsub(n))+1
      enddo
c
      call adjdt1(dtmax,nox,noy,nlayer,dtik,dtjk)
c      do 91 j=1,noy
c	write(*,'(a,i4,8f7.2)') 'j, dtsi :',j,(dtsi(j,k),k=1,nlayer)
c 91   continue
c      do 92 i=1,nox
c	write(*,'(a,i4,8f7.2)') 'i, dtsj :',i,(dtsj(i,k),k=1,nlayer)
c 92   continue
c
      write(*,3100) time,ndate,deltat,dtval,dt2
c      write(*,*) 'deltatk :',(deltatk(k),k=1,nlayer)
c      write(*,*) 'dtsub,nstps :',(dtsub(n),nstps(n),n=1,3)
      call flush(6)
c
3100  format(' TIME STEP SELECTION AT ',f10.1,i10,': ',3f10.2)
3101  format(' TIME STEP BY LAYER   ',i10,f10.1,i10,': ',4f8.2)
3102  format(' TIME STEP BY LAYER ADJUSTED ',i10,f10.1,i10,': ',4f8.2)
      return
      end
