c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine emcors(time,ndate,deltat,idebug)
c
c-----ATDM v1.00
c
c     Injects emissions into the coarse grid 
c
c-----95/02/22:
c     -from UAM-V Revision 1.2  94/02/10  20:55:02  tcm
c     -modified code for new PTSRCX and PTSRCY variables (see hedpts)
c     -removed call to CONVRT
c     -removed unused code that was commented out
c-----95/06/07:
c     -emissions converted from mol/hr to ppm rather than to umol/m3
c     -redefined checks of source grid location with respect to
c      boundary cells (e.g., cheking i.le.istr instead of i.lt.istr)
c
c     Input arguments:
c               time:           model time
c               ndate:          model date
c               deltat:         coarse grid time step (min)
c               idebug:         diagnostic debug flag
c     Files written:
c               6:              standard output file unit (*)     
c               8:              diagnostic file
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'com3.cmd'
      include 'deffin.cmd'
      include 'balanc.cmd'
      include 'stack.cmd'
c
      dimension  iatrac(mxspec), iptrac(mxspec)
      dimension  ilvtrc(mxz)
      real mfac
      data grav /9.8/, amolwt /28.8/
c
c  bit eleven of idebug on (i.e., the 1024's location) will turn
c  on debug writes in emcors.
c
      idiag = and(idebug,1024)
      if (idiag .ne. 0) then
        write(*,4000) time,ndate,deltat
      endif
      do 1 n=1,mxspec
        iatrac(n) = 0
        iptrac(n) = 0
    1 continue
      do 2 k=1,mxz
        ilvtrc(k) = 0
    2 continue
c
c     release point source emissions
c
      if (.not.lpts) goto 11
      do 10 l = 1,nopts
        i = int(ptsrcx(l)) + 1
        j = int(ptsrcy(l)) + 1
        if (i.gt.nox) i = nox
        if (i.lt.1) i = 1
        if (j.gt.noy) j = noy
        if (j.lt.1) j = 1
        if (i.le.istr(j)) goto 10
        if (i.ge.iend(j)) goto 10
        if (j.le.jstr(i)) goto 10
        if (j.ge.jend(i)) goto 10
        if (idiag .ne. 0) then
          write(*,*) 'nopts, l, i, j: ', nopts,l,i,j
        endif
c
        do 5 k = 1,nlayer
          if (heff(l).gt.h(i,j,k)) go to 5
c
c     put point source emissions in layer k
c
          dsigma = sigma(k) - sigma(k+1)
          ilvtrc(k) = ilvtrc(k) + 1
          vfac = deltat/(1.e6*deltay*deltax(j)*dsigma)
          mfac = 10.*grav*amolwt/pstar(i,j)
          do 4 lisp=1,nemit
            isp = kemit(lisp)
            if (isp .le. 0) go to 4
            if (isp .gt. nospec) stop117
            iptrac(isp) = iptrac(isp) + 1
            if (idiag .ne. 0) then
              write(*,*) 'isp, k:  ', isp, k
              write(*,*) 'deltat, temper:', deltat,
     &          temper(i,j,k)
              write(*,*) ' ptemis, press, deltay, deltax, hdel:',
     &          ptemis(lisp,l), press(i,j,k),deltay,deltax(j),
     &          hdel
              write(*,*) 'conc: ',conc(i,j,isp,k)
            endif
            conc(i,j,isp,k) = conc(i,j,isp,k)+ptemis(lisp,l)*vfac*mfac
            ptflow(isp) = ptflow(isp) + ptemis(lisp,l)*deltat
            if (isp .eq. kno2) then
              if (kno2s .gt. 0) then
                conc(i,j,kno2s,k) = conc(i,j,kno2s,k)
     &                                   +ptemis(lisp,l)*vfac*mfac
                ptflow(kno2s) = ptflow(kno2s) + ptemis(lisp,l)*deltat
              endif
            endif
            if (idiag .ne. 0) then
              write(*,*) 'after, conc: ',conc(i,j,isp,k)
            endif
    4     continue
          go to 10
    5   continue
   10 continue
c
c     release area emissions
c
   11 if (idiag .ne. 0) then
        write(*,*) ' about to do area sources, larea = ',larea
      endif
      if (.not.larea) go to 30
      if (idiag .ne. 0) then
        write(*,*) ' looping over grid, nox, noy = ',nox, noy
      endif
      do 26 j = 1,noy
        do 25 i = 1,nox
          if (i.le.istr(j)) goto 25
          if (i.ge.iend(j)) goto 25
          if (j.le.jstr(i)) goto 25
          if (j.ge.jend(i)) goto 25
          dsigma = 1. - sigma(2)
          vfac = deltat/(1.e6*deltay*deltax(j)*dsigma)
          mfac = 10.*grav*amolwt/pstar(i,j)
c
          do 20 lisp = 1,naemit
            isp = kaemit(lisp)
            if (isp .le. 0) go to 20
            if (tarea(isp,1).le.0.0) go to 20
            iatrac(isp) = iatrac(isp) + 1
            if (idiag .ne. 0) then
              if (i .gt. 9 .and. i .lt. 13) then
                if (j .gt. 9 .and. j .lt. 13) then
                  write(*,*) 'area source: isp, lisp:  ', isp, lisp
                  write(*,*) 'cell:  ',i,j
                  write(*,*) 'deltat, temper:', deltat,
     &                 temper(i,j,1)
                  write(*,*) ' qarea, press, deltay, deltax, h:',
     &                 qarea(i,j,lisp), press(i,j,1),deltay,deltax(j),
     &                 h(i,j,1)
                  write(*,*) 'conc: ',conc(i,j,isp,1)
                endif
              endif
            endif
            conc(i,j,isp,1) = conc(i,j,isp,1)+qarea(i,j,lisp)*vfac*mfac
            eflow(isp) = eflow(isp) + qarea(i,j,lisp)*deltat
            if (isp .eq. kno2) then
              if (kno2s .gt. 0) then
                conc(i,j,kno2s,1) = conc(i,j,kno2s,1)
     &                                  +qarea(i,j,lisp)*vfac*mfac
                eflow(kno2s) = eflow(kno2s) + qarea(i,j,lisp)*deltat
              endif
            endif
            if (idiag .ne. 0) then
              if (i .gt. 9 .and. i .lt. 13) then
                if (j .gt. 9 .and. j .lt. 13) then
                  write(*,*) 'after, conc: ',conc(i,j,isp,1)
                endif
              endif
            endif
   20     continue
   25   continue
   26 continue
c
   30 continue
c
      write(*,*) ' area sources: ',(iatrac(n),n=1,nospec)
      write(*,*) ' points: ',(iptrac(n),n=1,nospec)
      write(*,*) ' levels: ',(ilvtrc(n),n=1,nlayer)
      if (idiag .ne. 0) then
        write(*,4100) nopts 
      endif
      return
c
c 820 continue
c     write(8,3820) idy,iunit,time,ndate
c     write(*,3820) idy,iunit,time,ndate
c     stop1
c
 3820 format(' error emcors idy ',2i10,f10.1,i10)
 4000 format(' ENTER EMCORS ',f10.1,i10,f10.2)
 4100 format(' leave emcors',i10)
c
      end
