c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine fmscal(nxf,nyf,nzf,delfx,delfy,sigmaf,pstarf,
     &                  cncf,ifine)
c
c-----ATDM v1.00
c
c     Calculates mass budget for a fine grid
c
c-----95/06/07:
c     -from UAM-V Revision 1.2  94/02/10  20:56:08  tcm
c     -added presf and temprf to argument list
c     -conversion now from ppm to moles
c-----95/06/23:
c     -revised mfac and vfac for use with sigma rather than height
c
c     Input arguments:
c               nxf:            number of fine grid cells in x-direction       
c               nyf:            number of fine grid cells in y-direction       
c               nzf:            number of fine grid layers
c               delfx:          fine grid cell size in x-direction (km)
c               delfy:          fine grid cell size in y-direction (km)
c               sigmaf:         fine grid sigma structure
c               pstarf:         array of fine grid P* (mb)
c               cncf:           array of fine grid concentrations (ppm)
c               ifine:          fine grid index
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'balanc.cmd'
c
      dimension cncf(nxf,nyf,nospec,nzf), sigmaf(nzf+1)
      dimension delfx(nyf), pstarf(nxf,nyf)
      real mfac
      data grav /9.8/, amolwt /28.8/
c
      do 5535 l=1,nospec
        ftmas(l,ifine) = 0.
        do 5534 k=1,nzf
          do 5533 j=2,nyf-1
            do 5532 i=2,nxf-1
              dsigma = sigmaf(k) - sigmaf(k+1)
              vfac = 1.e6*delfx(j)*delfy*dsigma
              mfac = pstarf(i,j)/(10.*grav*amolwt)
              ftmas(l,ifine) = ftmas(l,ifine) + cncf(i,j,l,k)*vfac*mfac
 5532       continue
 5533     continue
 5534   continue
 5535 continue
      return
      end
