c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine gethp(nxx,nyy,nzz,nx,ny,nz,pmodtop,psfc,sigma,ch2o,
     &                 temper,pstar,press,ht)
c
c-----ATDM v1.00
c
c     Calculates pressure and layer interface height fields 
c     from surface pressure, temperature, and moisture fields for all grids
c
c-----95/06/02:
c     C Emery SAI
c-----95/08/24:
c     -added check to skip calculation for columns in which psfc < 0
c      (to allow for missing met data in non-modeled columns)
c
c     Input arguments: 
c               nxx:           max number of grid cells in x-direction    
c               nyy:           max number of grid cells in y-direction
c               nzz:           max number of layers
c               nx:            number of grid cells in x-direction    
c               ny:            number of grid cells in y-direction
c               nz:            number of layers
c               pmodtop:       model top pressure (mb)
c               psfc:          array of surface pressure (mb)
c               sigma:         array of sigma levels
c               ch2o:          array of water vapor (kg/kg)
c               temper:        array of temperature (K)
c     Output arguments:
c               pstar:         array of P* (mb)
c               press:         array of layer pressure (mb)
c               ht:            array of layer interface heights (m)
c
c     Called by:
c               update
c
      dimension psfc(nxx,nyy), sigma(nzz+1), ch2o(nxx,nyy,nzz),
     &          temper(nxx,nyy,nzz), pstar(nxx,nyy), 
     &          press(nxx,nyy,nzz), ht(nxx,nyy,nzz)
c
      data eps /0.622/, rd /287./, grav /9.8/
c
      do 200 i = 1,nx
        do 201 j = 1,ny
          if (psfc(i,j).lt.0.) goto 201
          pstar(i,j) = psfc(i,j) - pmodtop
          do 202 k = 1,nz
            prsupr = sigma(k+1)*pstar(i,j) + pmodtop
            prslwr = sigma(k)*pstar(i,j) + pmodtop
            press(i,j,k) = (prsupr + prslwr)/2.
            tvirt = temper(i,j,k)*(eps + ch2o(i,j,k))/
     &              (eps*(1. + ch2o(i,j,k)))
            hbar = rd*tvirt/grav
            if (k.eq.1) then
              ht(i,j,k) = hbar*alog(psfc(i,j)/prsupr)
            else
              ht(i,j,k) = ht(i,j,k-1) + hbar*alog(prslwr/prsupr)
            endif
  202     continue
  201   continue
  200 continue
c
      return
      end
