c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine getic(time,ndate,iunit,ieof)
c
c-----ATDM v1.00
c
c     Reads a time interval off a UAM type ic file and 
c     returns parameters 
c
c-----95/04/21:
c     -from UAM-V Revision 1.2  94/02/10  20:56:37  tcm
c-----96/10/18:DFG
c     -added temporary initial ozone concentration of 30 ppb for all cells.
c      It should be commented out if initial ozone concentrations are 
c      provided in Air Quality data.
c
c     Input arguments:
c               time:           model time   
c               ndate:          model date
c               iunit:          initial condition file unit number
c     Output arguments:
c               ieof:           end of file error flag
c     Files read:
c               iunit:          initial condition file
c     Files written:
c               6:              standard output file unit (*)
c     Routines called:
c               jtocal    caltoj
c     Called by:
c               setup
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'comuam.cmd'
      include 'ibccon.cmd'
      include 'comchem.cmd'
c
      dimension aq(mxx,mxy)
      character*4 mdum(10)
c
      ieof = 0
c
c  set all concentrations to numerical lower bound initially
c
      do 30 k=1,nlayer
        do 28 l=1,nospec
          do 26 j=1,noy
            do 24 i=1,nox
              conc(i,j,l,k) = bdnl(l)
   24       continue
   26     continue
   28   continue
   30 continue
c
c  find correct time and read initial concs
c
   50 read (iunit,end=900) idat, tdata, idate, tnext
      write(*,'(a,i8,f10.2,a,i8,f10.2,a,i4)')
     &     ' Read interval ',idat, tdata, ' to ', idate, tnext,
     &     ' on unit ', iunit
c
      do 200 l=1,nicsp
        lint = kicsp(l)
        do 190 k=1,nlayer
          read(iunit,end=905) nseg,(mdum(i),i=1,10),
     &                      ((aq(i,j),i=1,nox),j=1,noy)
          if (lint .eq. 0) go to 190
          do 150 j=1,noy
            do 140 i=1,nox
              conc(i,j,lint,k) = aq(i,j)
  140       continue
  150     continue
  190   continue
  200 continue
      if (kno2s .gt. 0) then
        if (kno2 .gt. 0) then
          do k=1,nlayer
            do j=1,noy
              do i=1,nox
                conc(i,j,kno2s,k) = conc(i,j,kno2,k)
              enddo
            enddo
          enddo
        endif
      endif
c
c***DFG
c set initial ozone concentration equal to 30 ppb.
c     do 44 j=1,noy
c       do 43 i=1,nox
c         do 42 k=1,nlayer-1
c         do 42 k=1,nlayer
c           conc(i,j,ko3,k)=30.0/1000.
c42       continue
c         conc(i,j,ko3,nlayer)=250./1000.
c43     continue
c44   continue
c***DFG
c
c  check time and if not correct, go back for another interval
c  (if ndate is calendar date, convert file date; but if ndate
c   is julian date, convert file date to julian)
c
      if (ndate .gt. 99999) then
        if (idat .lt. 99999) then
          call jtocal(idat,id1)
          call jtocal(idate,id2)
        else
          id1 = idat
          id2 = idate
        endif
      else
        if (idat .gt. 99999) then
          call caltoj(idat,id1)
          call caltoj(idate,id2)
        else
          id1 = idat
          id2 = idate
        endif
      endif
      tchk = tdata*100. + 2400.*(id1-ndate)
      tchk2 = tnext*100. + 2400.*(id2-ndate)
      if (time-tchk .lt. -0.1 .or. time-tchk2 .gt. 0.1) go to 50
c
      return
c
  900 ieof = 1
      return
c
  905 ieof = 1
      write(*,*) ' Unexpected end of file on IC file.'
      return
c
      end
