c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine getoh(atmf,oh0,o3p,zenith,temp,h2o,o3,cnox,voc,oh,
     &                 ierr)
c
c-----ATDM v1.00
c
c     Returns OH concentration from 1 of 2 input lookup tables given a set
c     of seven ambient parameters
c
c-----8/15/95:
c     A Kuklin and C Emery  SAI
c-----10/06/95:
c     -fixed some coding errors, will skip lookup if o3p = 0 (nighttime)
c
c     Input arguments:
c               atmf:           flag indicating lower or upper atmosphere 
c                               (1=lower atm, 2=upper atm)
c               oh0:            OH lookup table/vector (ppt)
c               o3p:            ozone photolysis rate (1/min)
c               zenith:         zenith angle (degrees)
c               temp:           temperature (K)
c               h2o:            water vapor concentration (ppm)
c               o3:             ozone concentration (ppm)
c               cnox:           NOx (NO+NO2) concentration (ppm)
c               voc:            VOC concentration (ppm)
c               ierr:           error flag
c     Output arguments:
c               oh:             hydroxyl concentration (ppm)
c     Routines called:
c               indexs    indexg
c     Called by:
c               rchem     rchemf
c
      include 'parchm.cmd'
c
      dimension oh0(nmoh)
      dimension o301(nmo3),o302(nmo3),h2o01(nmh2o),h2o02(nmh2o),
     &          voc01(nmvoc),voc02(nmvoc),cnox01(nmnox),cnox02(nmnox),
     &          t01(nmt),t02(nmt),z0(nmz),o3p01(nmo3p),o3p02(nmo3p)
      dimension imz(nmo3p)
c
c-----Reference values
c
      data o3p01/.00002,.00029,.00082,.0016,.0028,.0043/
      data o3p02/.00002,.0016,.003,.0041,.006,.0075/
      data z0/10.,40.,50.,60.,70./
      data t01/253.,263.,273.,283.,298.,313./
      data t02/213.,228.,243.,258.,273.,283./
      data h2o01/160.,1600.,8000.,14000.,20000.,26000.,32000./
      data h2o02/16.,160.,1600.,5600.,9600.,13600.,17600./
      data o301/.01,.02,.05,.1,.15,.2,.25,.3/
      data o302/.05,.1,.15,.2,.25,.3,.35,.4/
      data cnox01/.0001,.0005,.001,.0015,.002,.003,.005,.01,
     &            .02,.04,.06,.1,.2/
      data cnox02/.00003,.0001,.0005,.001,.0015,.002,.003,
     &            .005,.006,.007,.008,.01,.02/
      data voc01/.02,.05,.2,.3,.6,.9,1.5/
      data voc02/.01,.03,.05,.1,.2,.3,.4/
      data imz /5,4,3,2,1,1/
c
c-----Check for nighttime
c
      ierr = 0
      if (o3p.eq.0.) then
        oh = oh0(1)/1.e6
        return
      endif
c
      i0a = 1
      ia  = 1
      ja  = 1
      ka  = 1
      la  = 1
      ma  = 1
      na  = 1
      mal = 1
      nal = 1
c
c-----Get indices for solar zenith angle
c
      call indexs(zenith,z0,nmz,ial,ia)
c
c-----Get indices for lower atmosphere
c
      if (atmf .eq. 1.) then
c
        if (o3p.lt.o3p01(1))       o3p = o3p01(1)
        if (o3p.gt.o3p01(nmo3p))   o3p = o3p01(nmo3p)
        if (cnox.lt.cnox01(1))     cnox = cnox01(1)
        if (cnox.gt.cnox01(nmnox)) cnox = cnox01(nmnox)
        if (voc.lt.voc01(1))       voc = voc01(1)
        if (voc.gt.voc01(nmvoc))   voc = voc01(nmvoc)
c
        if (o3p .le. o3p01(1)) then
          i0al = 1
        else
          if (o3p .ge. o3p01(nmo3p)) then
            i0al = nmo3p - 1
          else
            call indexs(o3p,o3p01,nmo3p,i0al,i0a)
          endif
        endif
        call indexs(temp,t01,nmt,jal,ja)
        call indexs(h2o,h2o01,nmh2o,kal,ka)
        call indexs(o3,o301,nmo3,lal,la)
        if (cnox .le. cnox01(1)) then
          mal = 1
        else
          if (cnox .ge. cnox01(nmnox)) then
            mal = nmnox - 1
          else
            call indexs(cnox,cnox01,nmnox,mal,ma)
          endif
        endif
        if (voc .le. voc01(1)) then
          nal = 1
        else
          if (voc .ge. voc01(nmvoc)) then
            nal = nmvoc - 1
          else
            call indexs(voc,voc01,nmvoc,nal,na)
          endif
        endif
c       write (*, 110) o3p01(i0al),o3p01(i0al+1),z0(ia),t01(ja),
c    &                 h2o01(ka),o301(la),cnox01(mal),cnox01(mal+1),
c    &                 voc01(nal),voc01(nal+1)
c
c-----Get indices for upper atmosphere
c
      else
c
        if (o3p.lt.o3p02(1))       o3p = o3p02(1)
        if (o3p.gt.o3p02(nmo3p))   o3p = o3p02(nmo3p)
        if (cnox.lt.cnox02(1))     cnox = cnox02(1)
        if (cnox.gt.cnox02(nmnox)) cnox = cnox02(nmnox)
        if (voc.lt.voc02(1))       voc = voc02(1)
        if (voc.gt.voc02(nmvoc))   voc = voc02(nmvoc)
c
        if (o3p .le. o3p02(1)) then
          i0al = 1
        else
          if (o3p .ge. o3p02(nmo3p)) then
            i0al = nmo3p - 1
          else
            call indexs(o3p,o3p02,nmo3p,i0al,i0a)
          endif
        endif
        call indexs(temp,t02,nmt,jal,ja)
        call indexs(h2o,h2o02,nmh2o,kal,ka)
        call indexs(o3,o302,nmo3,lal,la)
        if (cnox .le. cnox02(1)) then
          mal = 1
        else
          if (cnox .ge. cnox02(nmnox)) then
            mal = nmnox - 1
          else
            call indexs(cnox,cnox02,nmnox,mal,ma)
          endif
        endif
        if (voc .le. voc02(1)) then
          nal = 1
        else
          if (voc .ge. voc02(nmvoc)) then
            nal = nmvoc - 1
          else
            call indexs(voc,voc02,nmvoc,nal,na)
          endif
        endif
c       write (*, 110) z0(ia),t02(ja),h2o02(ka),o302(la),cnox02(mal),
c    &                 cnox02(mal+1),voc02(nal),voc02(nal+1)
      endif
c
c-----Check solar zenith angle index against O3 photolysis index
c
      ia1 = ia
      if (i0al.eq.0) then
        write(*,*)'atmf,o3p,zenith,temp,h2o,o3,cnox,voc'
        write(*,*) atmf,o3p,zenith,temp,h2o,o3,cnox,voc
      endif
      if (ia .gt. imz(i0al)) then
        ia = imz(i0al)
        ia1 = ia
c       write (*,*) 'for given O3 photolisis rate ',o3p
c       write (*,*) 'the highest allowable zenith angle is ',
c    &              z0(imz(i0al))
c       write (*,*) 'GETOH is passed a zenith angle of ',zenith
      else
        if (ia .gt. imz(i0al+1)) then
          ia1 = imz(i0al+1)
c         write (*,*) 'for given O3 photolisis rate ',o3p
c         write (*,*) 'the highest allowable zenith angle is ',
c    &                z0(imz(i0al+1))
c         write (*,*) 'GETOH is passed a zenith angle of ',zenith
        endif
      endif
c     write (*, 120) i0al,i0al+1,ia,ja,ka,la,mal,mal+1,nal,nal+1
c
c-----Determine table indices for lookup/interpolation
c
      call indexg (i0al,ia,ja,ka,la,mal,nal,ind1)
      call indexg (i0al+1,ia1,ja,ka,la,mal,nal,ind2)
      call indexg (i0al,ia,ja,ka,la,mal+1,nal,ind3)
      call indexg (i0al,ia,ja,ka,la,mal,nal+1,ind4)
      call indexg (i0al,ia,ja,ka,la,mal+1,nal+1,ind5)
      call indexg (i0al+1,ia1,ja,ka,la,mal,nal+1,ind6)
      call indexg (i0al+1,ia1,ja,ka,la,mal+1,nal,ind7)
      call indexg (i0al+1,ia1,ja,ka,la,mal+1,nal+1,ind8)
c
c-----Tri-linear interpolation procedure
c
      if (mal.eq.0) then
        write(*,*)'cnox,voc,o3p',cnox,voc,o3p
        ierr = 1
        return
      endif
      if (atmf .eq. 1.) then
        t = (o3p - o3p01(i0al))/(o3p01(i0al+1) - o3p01(i0al))
        u = (cnox - cnox01(mal))/(cnox01(mal+1) - cnox01(mal))
        v = (voc - voc01(nal))/(voc01(nal+1) - voc01(nal))
      else
        t = (o3p - o3p02(i0al))/(o3p02(i0al+1) - o3p02(i0al))
        u = (cnox - cnox02(mal))/(cnox02(mal+1) - cnox02(mal))
        v = (voc - voc02(nal))/(voc02(nal+1) - voc02(nal))
      endif
c
      oh = ((1-t)*(1-u)*(1-v)*oh0(ind1) + t*(1-u)*(1-v)*oh0(ind2) +
     &     (1-t)*u*(1-v)*oh0(ind3) + (1-t)*(1-u)*v*oh0(ind4) +
     &     (1-t)*u*v*oh0(ind5) + t*(1-u)*v*oh0(ind6) +
     &     t*u*(1-v)*oh0(ind7) + t*u*v*oh0(ind8))/1000000.
c
c     write (*,*)'     oh   oh0(ind1) oh0(ind2) oh0(ind3) oh0(ind4) oh0
c    & (ind5) oh0(ind6) oh0(ind7) oh0(ind8)'
c     write (*,130)oh,oh0(ind1),oh0(ind2),oh0(ind3),oh0(ind4),oh0(ind5)
c    & ,oh0(ind6),oh0(ind7),oh0(ind8)
c
      return
c
 110  format (10(e9.3,1x))
 120  format (10(4x,i2,4x))
 130  format (9(e9.3,1x))
 140  format (9(i9,1x))
      end
c
c-----------------------------------------------------------------------
c
      subroutine indexs(par,x,imax,indl,ind)
c
      dimension x(imax)
c
      il = 0
      iu = imax+1
  10  if (iu-il .gt.1) then
        im = (iu+il)/2
        if (par .gt. x(im)) then
          il = im
        else
          iu = im
        endif
        goto 10
      endif
c
      indl = il
      if (indl .eq. 0) then
        ind = 1
      else
        if (indl .eq. imax) then
          ind = imax
        else
          if (abs(par-x(indl)) .le. abs(par-x(indl+1))) then
            ind = indl
          else
            ind = indl + 1
          endif
        endif
      endif
c
      return
      end
c
c-----------------------------------------------------------------------
c
      subroutine indexg(i0,i,j,k,l,m,n,ind)
c
      include 'parchm.cmd'
      dimension i0m(nmo3p)
c
      data i0m /0,5,9,12,14,15/
      ind = (i0m(i0)+i-1)*n5 + (j-1)*n4 + (k-1)*n3 + (l-1)*n2 +
     &      (m-1)*n1 + n
c
      return
      end
