c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine hedo3(time,ndate)
c
c-----ATDM v5.00
c
c     Reads REMSAD instantaneous of O3,OH,no3,h2o2,ho2 file when it is necesary
c
c-----99/04/15:
c
c     Input arguments:
c               time:           model time
c               ndate:          model date
c     Files read:
c               7:              ATDM control file
c             io3:        coarse grid concentration file for O3,OH,no3,h2o2,ho2
c            io3f:        fine grid concentration file for O3,OH,no3,h2o2,ho2
c     Files written:
c               6:              standard output file unit (*)
c               8:              diagnostic file
c               66:             output message file
c     Routines called:
c               caltoj    fgread
c     Called by:
c               setup
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
c
      character*80 fn, msgrs
      character*10 mchk1, mchk2, mchk3, mchk4, mchk5, mchk6, mchk7
      character*4 mrsspc(10,7)
      character*4 ifile(10), ifid(60)
      nskip = 0
c
      if ( ndate .gt. 99999) then
        call caltoj(ndate,nn)
        ndate = nn
      endif
c
c  read headers
c
      read(io3) (ifile(i),i=1,10),(ifid(i),i=1,60),ndum,
     &             nspio3,idd1,tb,idd2,te
      read(io3) f1,f2, utmzon, xu, yu, dx, dy,
     &          nx2, ny2, nl2, i1, i2, f3, f4, f5
      if (nl2 .ne. nlayer .or. nspio3 .lt. 5) then
        write(*,'(a,/,a,2i5,/,a,2i5)') 
     &       ' bad number of layers or species on O3 OH no3 h2o2 ho2 file',
     &       ' on O3 OH no3 h2o2 ho2 file: ',nl2, nspio3,
     &       ' in current run: ',nlayer
        write(8,'(a,/,a,2i5,/,a,2i5)') 
     &       ' bad number of layers or species on O3 OH no3 h2o2 ho2 file',
     &       ' on O3 OH no3 h2o2 ho2 file: ',nl2, nspio3,
     &       ' in current run: ',nlayer
        stop
      endif
      if (nx2 .ne. nox .or. ny2 .ne. noy) then
        write(*,'(a,/,a,2i5,/,a,2i5)') 
     &       ' bad x/y dimensions on O3 OH no3 h2o2 ho2 file',
     &       ' on O3 OH no3 h2o2 ho2 file: ',nx2, ny2,
     &       ' in current run: ',nox, noy
        write(8,'(a,/,a,2i5,/,a,2i5)') 
     &       ' bad x/y dimensions on O3 OH no3 h2o2 ho2 file',
     &       ' on O3 OH no3 h2o2 ho2 file: ',nx2, ny2,
     &       ' in current run: ',nox, noy
        stop
      endif
      read (io3)  i1,i2,ndum,ndum2
      read (io3) (mrsspc(m,1),m=1,10)
      read (io3) (mrsspc(m,2),m=1,10)
      read (io3) (mrsspc(m,3),m=1,10)
      read (io3) (mrsspc(m,4),m=1,10)
      read (io3) (mrsspc(m,5),m=1,10)
      read (io3) (mrsspc(m,6),m=1,10)
      if (nspio3 .gt. 6) read (io3) (mrsspc(m,7),m=1,10)
c     write(*,'(10A1)') ((mrsspc(i,l),i=1,10),l=1,nspio3)
      write(*,'(10A1)') ((mrsspc(i,l),i=1,10),l=1,7)
      write(mchk1,'(10a1)') (mrsspc(i,1),i=1,10)
      write(mchk2,'(10a1)') (mrsspc(i,2),i=1,10)
      write(mchk3,'(10a1)') (mrsspc(i,3),i=1,10)
      write(mchk4,'(10a1)') (mrsspc(i,4),i=1,10)
      write(mchk5,'(10a1)') (mrsspc(i,5),i=1,10)
      write(mchk6,'(10a1)') (mrsspc(i,6),i=1,10)
      if (nspio3 .gt. 6) write(mchk7,'(10a1)') (mrsspc(i,7),i=1,10)
      call flush(6)

      if (mchk1 .ne. 'OH        ') then
        write(*,'(a)') 
     &   ' coarse grid OH species do not match'
        write(*,'(a10)') mchk1
        write(8,'(a)') 
     &   ' coarse grid OH species do not match'
        stop
      endif
      if (mchk2 .ne. 'HO2       ') then
        write(*,'(a)') 
     &   ' coarse grid HO2 species do not match current O3 species'
        write(*,'(a10)') mchk2
        write(8,'(a)')
     &   ' coarse grid HO2 species do not match current O3 species'
        stop
      endif
      if (mchk3 .ne. 'NO3       ') then
        write(*,'(a)') 
     &   ' coarse grid NO3 species do not match'
        write(*,'(a10)') mchk3
        write(8,'(a)') 
     &   ' coarse grid NO3 species do not match'
        stop
      endif
c     if (mchk4 .ne. 'ASO2      ') then
c       write(*,'(a)') 
c    &   ' coarse grid ASO2 species do not match'
c       write(*,'(a10)') mchk4
c       write(8,'(a)') 
c    &   ' coarse grid ASO2 species do not match'
c       stop
c     endif
      if (mchk5 .ne. 'H2O2      ') then
        write(*,'(a)') 
     &   ' coarse grid H2O2 species do not match'
        write(*,'(a10)') mchk5
        write(8,'(a)') 
     &   ' coarse grid H2O2 species do not match'
        stop
      endif
      if (mchk6 .ne. 'O3        ') then
        write(*,'(a)') 
     &   ' coarse grid O3 species do not match'
        write(*,'(a10)') mchk6
        write(8,'(a)') 
     &   ' coarse grid O3 species do not match'
        stop
      endif
      if (nspio3 .gt. 6) then
        if (mchk7 .ne. 'PEC       ') then
          write(*,'(a)') 
     &   ' coarse grid PEC species do not match'
          write(*,'(a10)') mchk7
          write(8,'(a)') 
     &   ' coarse grid PEC species do not match'
          stop
        endif
      endif

   15 continue

      if (numfin .gt. 0) then
        read(io3f) msgrs
        write(*,'(a,/,2x,a)') ' fine grid O3 OH no3 h2o2 ho2 file labelled:',
     &                          msgrs
        write(8,'(a,/,2x,a)') ' fine grid O3 OH no3 h2o2 ho2 file labelled:',
     &                          msgrs
        read(io3f) nrsfin, nrsspc
        if (nrsfin .ne. numfin .or. nrsspc .ne. 7) then
          write(*,'(a,/,a,2i5,/,a,2i5)') 
     &         ' bad number of fine grids or species on O3 OH no3 h2o2 ho2 file',
     &         ' on O3 oh no3 h2o2 ho2  file: ',nrsfin,nrsspc,
     &         ' in current run: ',numfin
          write(8,'(a,/,a,2i5,/,a,2i5)') 
     &         ' bad number of fine grids or species on O3 OH no3 h2o2 ho2 file',
     &         ' on O3 OH no3 h2o2 ho2 file: ',nrsfin,nrsspc,
     &         ' in current run: ',numfin
          stop
        endif
        read(io3f) (mrsspc(i,1),i=1,10)
        read(io3f) (mrsspc(i,2),i=1,10)
        read(io3f) (mrsspc(i,3),i=1,10)
        read(io3f) (mrsspc(i,4),i=1,10)
        read(io3f) (mrsspc(i,5),i=1,10)
        read(io3f) (mrsspc(i,6),i=1,10)
        read(io3f) (mrsspc(i,7),i=1,10)
c
        write(mchk1,'(10a1)') (mrsspc(i,1),i=1,10)
        write(mchk2,'(10a1)') (mrsspc(i,2),i=1,10)
        write(mchk3,'(10a1)') (mrsspc(i,3),i=1,10)
        write(mchk4,'(10a1)') (mrsspc(i,4),i=1,10)
        write(mchk5,'(10a1)') (mrsspc(i,5),i=1,10)
        write(mchk6,'(10a1)') (mrsspc(i,6),i=1,10)
        write(mchk7,'(10a1)') (mrsspc(i,7),i=1,10)
        if (mchk1 .ne. 'OH        ') then
          write(*,'(a)')
     &     ' fine grid OH species do not match'
          write(*,'(a10)') mchk1
          write(8,'(a)')
     &     ' fine grid OH species do not match'
        endif
        if (mchk2 .ne. 'HO2       ') then
          write(*,'(a)')
     &     ' fine grid HO2 species do not match'
          write(*,'(a10)') mchk2
          write(8,'(a)')
     &     ' fine grid HO2 species do not match'
        endif
        if (mchk3 .ne. 'NO3       ') then
          write(*,'(a)')
     &     ' fine grid NO3 species do not match'
          write(*,'(a10)') mchk3
          write(8,'(a)')
     &     ' fine grid NO3 species do not match'
        endif
        if (mchk4 .ne. 'ASO2      ') then
          write(*,'(a)')
     &     ' fine grid ASO2 species do not match'
          write(*,'(a10)') mchk4
          write(8,'(a)')
     &     ' fine grid ASO2 species do not match'
        endif
        if (mchk5 .ne. 'H2O2      ') then
          write(*,'(a)')
     &     ' fine grid H2O2 species do not match'
          write(*,'(a10)') mchk5
          write(8,'(a)')
     &     ' fine grid H2O2 species do not match'
        endif
        if (mchk6 .ne. 'O3        ') then
          write(*,'(a)') 
     &     ' fine grid O3 species do not match current O3 species'
          write(*,'(a10)') mchk6
          write(8,'(a)') 
     &     ' fine grid O3 species do not match current O3 species'
        endif
        if (mchk7 .ne. 'PEC       ') then
          write(*,'(a)') 
     &     ' fine grid PEC species do not match current PEC species'
          write(*,'(a10)') mchk7
          write(8,'(a)') 
     &     ' fine grid PEC species do not match current PEC species'
        endif

        do 18 nn=1,nrsfin
          read(io3f) ixfbrs, jyfbrs,
     &             ixfers, jyfers, nhfrs,
     &             nvfrs, nfxrs, nfyrs, 
     &             nfzrs
          if (ixfbrs .ne. ixfb(nn) .or.
     &        jyfbrs .ne. jyfb(nn) .or.
     &        ixfers .ne. ixfe(nn) .or.
     &        jyfers .ne. jyfe(nn) .or.
     &        nhfrs .ne. nhf(nn) .or.
     &        nvfrs .ne. nvf(nn) .or.
     &        nfxrs .ne. nfx(nn) .or.
     &        nfyrs .ne. nfy(nn) .or.
     &        nfzrs .ne. nfz(nn)) then
            write(*,'(a,a)') 'O3 OH NO3 H2O2 HO2 and current fine grid ',
     &          'definitions do not match'
            write(*,'(a,9i5)') 'O3 OH NO3 H2O2 HO2: ', ixfbrs, jyfbrs, ixfers,
     &           jyfers, nhfrs, nvfrs, nfxrs, nfyrs, nfzrs
            write(*,'(a,9i5)') 'current: ', ixfb(nn), jyfb(nn), 
     &           ixfe(nn), jyfe(nn), nhf(nn), nvf(nn), nfx(nn), 
     &           nfy(nn), nfz(nn)
c
            write(8,'(a,a)') 'O3 OH NO3 H2O2 HO2 and current fine grid ',
     &          'definitions do not match'
            write(8,'(a,9i5)') 'O3 OH NO3 H2O2 HO2: ', ixfbrs, jyfbrs, ixfers,
     &           jyfers, nhfrs, nvfrs, nfxrs, nfyrs, nfzrs
            write(8,'(a,9i5)') 'current: ', ixfb(nn), jyfb(nn), 
     &           ixfe(nn), jyfe(nn), nhf(nn), nvf(nn), nfx(nn), 
     &           nfy(nn), nfz(nn)
            call flush(6)
            stop
          endif
   18   continue
      endif
c
      return
      end
