c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
c
      subroutine hedqar(nosp, nxsz, nysz, iunit, nsem, kespc,
     &                  katol, kaxyl)
c
c-----ATDM v1.00
c
c     Reads header of UAM area source emissions file and
c     fills in parameters for main routine
c
c-----95/02/22:
c     -from UAM-V Revision 1.2  94/02/10  20:57:10  tcm
c     -removed code specific to FORM and HONO
c
c     Input arguments
c     Input arguments:
c               nosp:           number of modeled species
c               nxsz:           number of grid cells in x-direction
c               nysz:           number of grid cells in y-direction
c               iunit:          area source file unit number
c     Output arguments:
c               nsem:           number of emitted species
c               kespc:          array to map emitted species to modeled species
c               katol:          index to toluene
c               kaxyl:          index to xylene
c     Files read:
c               iunit:          area source file
c     Files written:
c               6:              standard output file unit (*)
c     Called by:
c               atdm      setup

c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'comchem.cmd'
c
      dimension kespc(mxemit)
c
      character*4 memsp(10,mxemit), note(60), ifile(10) 
cae      dimension mdum(10)
      character*10 m1, m2
c
      read(iunit) ifile, note, nseg, nemspc, idate, begtim, 
     &            jdate, endtim
      write(m1,'(10a1)') ifile
      if (m1 .ne. 'EMISSIONS ') then
        write(*,'(a,i2,a,a,/,a)') ' File on unit ',iunit,' is ',m1,
     &           ' It should be EMISSIONS.'
        stop
      endif
      if (nemspc .gt. mxemit) then
        write(*,*) 'NUMBER OF SPECIES ON INPUT FILE (',nemspc,')',
     &             ' IS GREATER THAN MAX ALLOWED (',mxemit,').'
        stop
      endif
      read  (iunit) orgx, orgy, izone, utmx, utmy, deltax, deltay,
     &          nx, ny, nz, nzlowr, nzuppr, htsur, htlow, htupp
      if (nx .ne. nxsz) then
        write(*,*) 'NUMBER OF X CELLS ON INPUT FILE (',nx,')',
     &             ' IS INCORRECT. (',nxsz,').'
        stop
      endif
      if (ny .ne. nysz) then
        write(*,*) 'NUMBER OF Y CELLS ON INPUT FILE (',ny,')',
     &             ' IS INCORRECT. (',nysz,').'
        stop
      endif
      read(iunit) nxx,nyy,nxcell,nycell
      read(iunit) ((memsp(m,l),m=1,10),l=1,nemspc)
c
c  find each species on emissions file in standard species list
c  (0 will mean a species on emissions file that is not needed.)
c
      do 100 l=1,nemspc
        write(m1,'(10a1)') (memsp(i,l),i=1,10)
        if (m1 .eq. 'TOL       ') katol = l
        if (m1 .eq. 'XYL       ') kaxyl = l
        do 80 lint=1,nosp
          write(m2,'(10a1)') (mspec(i,lint),i=1,10)
          if (m1 .eq. m2) then
            kespc(l) = lint
            write(*,'(a,i3,1x,a,a,i3,1x,a,a)') ' Species ',l,m1,
     &            ' on EMISSIONS file = species ',lint,m2,' internally.'
            go to 100
          endif
   80   continue
        kespc(l) = 0
  100 continue
c
      nsem = nemspc
      return
      end
