c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine nstfil(cnc1,cnc2,dry1,dry2,wet1,wet2,pstrf1,pstrf2,
     &                  sigf1,sigf2,nxf1,nyf1,nzf1,nxf2,nyf2,nzf2,nsp,
     &                  if1,if2)
c
c-----ATDM v1.00
c
c     Fills in fine grid concentrations with other nested fine grid
c     concentrations.  Should be done at end of time step
c     after all transport and chem calculations are done.
c
c-----95/03/31:
c     -from UAM-V Revision 1.2  94/02/10  20:58:27  tcm
c     -modifications made following UAM-V to correct vertical nesting code
c-----95/06/20:
c     -revised to fill coarser grid with ppm rather than umol/m3
c     -added prsf1,2 and tmpf1,2 to argument list
c-----95/06/23:
c     -revised mfac and vfac for use with sigma rather than height
c     -removed tmpf1,2, prsf1,2, and htf1,2 from argument list, added sigf1,2
c      and pstrf1,2
c     -added data statements for grav and amolwt
c-----95/11/08:
c     -added fill of dpositf and wpositf with respective finer grid amounts
c-----95/11/28:
c     -fixed some coding errors related to entry above
c
c     Input arguments:
c               cnc1:           finer grid concentration (ppm)   
c               cnc2:           coarser grid concentration (ppm)
c               dry1:           finer grid dry dep flux (g/km2)
c               dry2:           coarser grid dry dep flux (g/km2)
c               wet1:           finer grid wet dep flux (g/km2)
c               wet2:           coarser grid wet dep flux (g/km2)
c               pstrf1:         finer grid P* (mb)
c               pstrf2:         coarser grid P* (mb)
c               sigf1:          finer grid sigma structure
c               sigf2:          coarser grid sigma structure
c               nxf1:           finer grid number of cells in x-direction
c               nyf1:           finer grid number of cells in y-direction
c               nzf1:           finer grid number of layers
c               nxf2:           coarser grid number of cells in x-direction
c               nyf2:           coarser grid number of cells in y-direction
c               nzf2:           coarser grid number of layers
c               nsp:            number of modeled species
c               if1:            finer grid index
c               if2:            coarser grid index
c     Output arguments:
c               cnc2:           coarser grid concentration (ppm)
c               dry2:           coarser grid dry dep flux (g/km2)
c               wet2:           coarser grid wet dep flux (g/km2)
c     Files written:
c               6:              standard output file unit (*)     
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'comuam.cmd'
      include 'deffin.cmd'
      include 'ctlflg.cmd'
c
      dimension cnc1(nxf1,nyf1,nsp,nzf1)
      dimension cnc2(nxf2,nyf2,nsp,nzf2)
      dimension dry1(nxf1,nyf1,nsp),wet1(nxf1,nyf1,nsp)
      dimension dry2(nxf2,nyf2,nsp),wet2(nxf2,nyf2,nsp)
      dimension pstrf1(nxf1,nyf1)
      dimension pstrf2(nxf2,nyf2)
      dimension sigf1(nzf1+1), sigf2(nzf2+1)
      real mfac
      data grav /9.8/, amolwt /28.8/
c
c  set up some parameters relating the two grids
c
      nvr = nvf(if1)/nvf(if2)
      if (nvr .le. 0) then
        write(*,*) ' coarse vertical structure within fine ',
     &             ' vertical structure.',
     &             ' error in nstfil.'
        write(*,*) 'if1, if2: ',if1,if2
        stop
      endif
      ixff0 = (ixfb(if1) - ixfb(if2))*nhf(if2) + 2
      ixff1 = (ixfe(if1) - ixfb(if2) + 1)*nhf(if2) + 1
      jyff0 = (jyfb(if1) - jyfb(if2))*nhf(if2) + 2
      jyff1 = (jyfe(if1) - jyfb(if2) + 1)*nhf(if2) + 1
      if (ixff0 .lt. 2 .or. ixff1 .ge. nxf2 .or.
     &    jyff0 .lt. 2 .or. jyff1 .ge. nyf2) then
        write(*,*) ' erroneous grid parameters in nstfil'
        write(*,*) ' ixff0, ixff1, jyff0, jyff1: ',ixff0,ixff1,
     &               jyff0, jyff1
        write(*,*) 'if1, if2: ',if1,if2
        stop
      endif
c
      nhr = nhf(if1)/nhf(if2)
      if (nhr .le. 0) then
        write(*,*) ' coarse horizontal structure within fine ',
     &             ' horizontal structure.',
     &             ' error in nstfil.'
        write(*,*) 'if1, if2: ',if1,if2
        stop
      endif
c
c  zero out coarse grid where overlapped by this fine grid
c
      do 100 k=1,nzf2
        do 90 n=1,nsp
          do 80 j=jyff0,jyff1
            do 70 i=ixff0,ixff1
              cnc2(i,j,n,k) = 0.
              if (ldepn .and. k.eq.1) dry2(i,j,n) = 0.
              if (lwet .and. k.eq.1) wet2(i,j,n) = 0.
   70       continue
   80     continue
   90   continue
  100 continue 
c
c  sum coarse grid for all levels and all species
c
      do 200 k=1,nzf2
        do 190 n=1,nsp
          kf2 = kvmap(k,if1)
          if (k .eq. 1) then
            kf1 = 1
          else
            kf1 = kvmap(k-1,if1) + 1
          endif
          do 180 kf=kf1,kf2
            dsigma = sigf1(kf) - sigf1(kf+1)
            do 150 j=2,nyf1-1
              jc = jyff0 + (j-2)/nhr
              do 140 i=2,nxf1-1
                vfac = 1.e6*delfx(j,if1)*delfy(if1)*dsigma
                mfac = pstrf1(i,j)/(10.*grav*amolwt)
                ic = ixff0 + (i-2)/nhr
                cnc2(ic,jc,n,k) = cnc2(ic,jc,n,k)
     &                  + cnc1(i,j,n,kf)*vfac*mfac
                if (ldepn .and. kf.eq.1) then
                  dry2(ic,jc,n) = dry2(ic,jc,n) +
     &                  delfx(j,if1)*delfy(if1)*dry1(i,j,n)
                endif
                if (lwet .and. kf.eq.1) then
                  wet2(ic,jc,n) = wet2(ic,jc,n) +
     &                  delfx(j,if1)*delfy(if1)*wet1(i,j,n)
                endif
  140         continue
  150       continue
  180     continue
  190   continue
  200 continue
c
c  recalculate conc as concentration instead of mass
c
      do 300 k=1,nzf2
        dsigma = sigf2(k) - sigf2(k+1)
        do 290 n=1,nsp
          do 280 j=jyff0,jyff1
            do 270 i=ixff0,ixff1
              vfac = 1.e6*delfx(j,if2)*delfy(if2)*dsigma
              mfac = pstrf2(i,j)/(10.*grav*amolwt)
              cnc2(i,j,n,k) = cnc2(i,j,n,k)/vfac/mfac
              if (ldepn .and. k.eq.1) then
                dry2(i,j,n) = dry2(i,j,n)/(delfx(j,if2)*delfy(if2))
              endif
              if (lwet .and. k.eq.1) then
                wet2(i,j,n) = wet2(i,j,n)/(delfx(j,if2)*delfy(if2))
              endif
  270       continue
  280     continue
  290   continue
  300 continue
c
      return
      end
