c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine partchm(pm,temper,presur,h2o,rh,o3amb,rkno2,oh,zcoef,
     &                   deltat,lprnt)
c
c-----ATDM v1.00
c
c     Calculates transformation of SO2 to sulfate and NOx to nitrate
c
c-----95/03/06:
c     -from RTM-II 93/04/19, adapted from RIVAD chemical routine within
c      ARM3-1.0, November 1987
c     -Restructured the code for better flow, deleted unused variables and
c      code
c     -Altered input of RH to input of H2O -- RH calculated internally
c     -Directed aqueous and heterogeneous sulfate formation to ASO4 species 
c      separately; gas-phase OH sulfate formation to GSO4 species.
c     -Concentrations now coming in as ppm
c-----95/10/30:
c     -added NO titration to NO2 at night for NO < O3
c
c     Input arguments:
c               pm              concentration of 7 species (ppm)
c                               SO2, GSO4, ASO4, NO, NO2, VOC, HNO3
c               temper          ambient temperature (K)
c               presur          ambient pressure (mb)
c               h2o             ambient water vapor concentration (ppm)
c               rh              ambient relative humidity (%)
c               o3amb           background ozone concentration (ppm)
c               rkno2           no2 photolysis rate (1/ppm-min)
c               oh              hydroxyl radical concentration (ppm)
c               zcoef           cosine of solar zenith angle
c               deltat          time step (min)
c               lprnt           diagnostic debug print flag 
c     Output arguments:
c               pm              concentration of 7 species (ppm)
c     Called by:
c               rchem     rchemf
c
      save
      dimension pm(7)
      logical lprnt
      data r26 /26.4/
      data rkvoc10 /2.3e+3/
      data rkvoc20 /7.4e-5/, tdepvoc2 /2000./
      data rkvoc30 /5.7e-1/
c
      tstp = deltat/60.
      tfac = 1./298. - 1./temper
      if (rh.gt.95.) rh = 95.
      if (rh.lt.0.) rh = 0.
      if (lprnt) then
        write(*,*)'chmriv: pm'
        write(*,*)(pm(l),l=1,7)
        write(*,*)'chmriv: temper,presur,h2o,rh,o3amb,rkno2,zcoef',
     &            'oh,tstp'
        write(*,*) temper,presur,h2o,rh,o3amb,rkno2,zcoef,oh,tstp
      endif
c
      cno = pm(4)
      cno2 = pm(5)
      cnox = cno + cno2
      if (rkno2.le.0.0) then
         phikk = 0.0
      else
         phikk = rkno2/r26
      endif
      if (lprnt) write(*,*)'phikk,cnox',phikk,cnox
c
c-----Do simple NOx-O3 equilibrium chemistry
c
      xoh = oh
      xnox = cnox
      xox = o3amb + cno2
      sum = xox + xnox + phikk
      if (phikk.le.0. .and. o3amb.gt.cno) then
        xno2 = xnox
      else
        xno2 = 0.5*(sum - sqrt(abs(sum*sum - 4.*xnox*xox)))
      endif
      if (xno2.gt.xnox) xno2 = xnox
      if (xno2.lt.0.) xno2 = 0.
      xno = xnox - xno2
      if (xno.lt.0.0) xno = 0.0
      rno2x = 0.
      if (xnox.gt.0.) rno2x = xno2/xnox
      xo3 = xox - xno2
      if (xo3.lt.0.) xo3 = 0.
      if (lprnt) write(*,*)'xno,xno2,xo3',xno,xno2,xo3
c
c-----Decay VOC via O3
c
      rkvoc2 = rkvoc20*exp(tdepvoc2*tfac)
      rvoc = rkvoc2*xo3*deltat
      xvoc = pm(6)*exp(-rvoc)
      if (lprnt) write(*,*)'rkvoc2,rvoc,xvoc',rkvoc2,rvoc,xvoc
c
c-----Branch to nighttime or daytime chemistry
c
      if (zcoef.ge.0.06975 .or. xo3.eq.0.) go to 56
c
c-----Do nighttime chemistry
c
      rsulf = 0.
      rnitr = 0.0
      rkx = 1780./(1.9e-6*h2o + 3.12)
      rno3 = 0.086
      a0 = 0.59 + 1780. - 3.12*rkx
      a = 2.*rkx*rno3 - a0
      b = rno3 + 0.0474*xo3 + xno2*a0
      c = -0.0474*xno2*xo3
      xno3 = -2.*c/(b + sqrt(b*b - 4.*a*c))
c
      rvoc = rkvoc30*xno3*deltat
      xvoc = xvoc*exp(-rvoc)
      xn2o5 = xno3*xno2*rkx
      rnitrn = 2.*1.9e-6*h2o*xn2o5*deltat
      if (rnitrn.gt.xno2) rnitrn = xno2
      pm(7) = pm(7) + rnitrn
      xno2 = xno2 - rnitrn
      if (lprnt) write(*,*)'NIGHT:rsulf,rnitr,rvoc,xvoc,rnitrn,xno2'
      if (lprnt) write(*,*) rsulf,rnitr,rvoc,xvoc,rnitrn,xno2
      go to 66
c
c-----Do daytime chemistry
c
   56 continue
      rsulf = 1.2e5*xoh*tstp
      rnitr = 8.4e5*xoh*tstp
      rvoc = rkvoc10*xoh*deltat
      xvoc = xvoc*exp(-rvoc)
      if (lprnt) write(*,*)'DAY:rsulf,rnitr,rvoc,xvoc'
      if (lprnt) write(*,*) rsulf,rnitr,rvoc,xvoc 
c
      rnitrd = rnitr*xno2
      if (rnitrd.gt.xnox) rnitrd = xnox
      pm(7) = pm(7) + rnitrd
      rnitro = tstp*xoh*(2.e6*xno + 1.e5*xno2)
      xnox = xnox-rnitrd - rnitro
      if (xnox.lt.0.) xnox = 0.
      xno = xnox*(1. - rno2x)
      xno2 = xnox*rno2x
      if (lprnt) write(*,*)'rnitrd,rnitro,xno,xno2'
      if (lprnt) write(*,*) rnitrd,rnitro,xno,xno2
c
c-----Sulfate chemistry combines slow heterogeneous reactions (bsulf),
c     aqueous reactions (wsulf) and OH reactions (rsulf)
c
   66 bsulf = 0.002*tstp 
      wsulf = 0.0
      if (rh.gt.70.) wsulf = tstp*0.01*(exp(0.072*(rh - 70.)) - 1.)
      sulfw = pm(1)*(wsulf + bsulf)
      if (sulfw.gt.pm(1)) sulfw = pm(1)
      pm(3) = pm(3) + sulfw
      pm(1) = pm(1) - sulfw
      sulfg = pm(1)*rsulf
      if (sulfg.gt.pm(1)) sulfg = pm(1)
      pm(2) = pm(2) + sulfg
      pm(1) = pm(1) - sulfg
      if (lprnt) write(*,*)'bsulf,wsulf,sulfw,sulfg'
      if (lprnt) write(*,*) bsulf,wsulf,sulfw,sulfg 
c
c     update new steady-state no and no2 values
c
      pm(4) = xno
      pm(5) = xno2
      pm(6) = xvoc

      if (lprnt) then
        write(*,*) 'chmriv: pm'
        write(*,*) (pm(l),l=1,7)
      endif
      return
      end
