c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine partdep(rsize,rmass,nbin,ustar,zo,rho,temp,
     &                   ra,rhoair,vmean)
c
c-----ATDM v1.00
c
c     Estimates a mass weighted mean deposition velocity
c     for a defined particulate distribution.  The method that is used
c     is similar to that used in ADOM.  The method assumes no
c     resuspension from vegetation surfaces.
c
c-----95/04/03:
c     -from UAM-V Revision 1.2  94/02/10  20:58:31  tcm
c     -Cleaned up comments
c
c     Input arguments:
c               rsize:          array of particle sizes (microns)
c               rmass:          array of mass fraction for each particle size
c               nbin:           number of particle size bins
c               ustar:          true neutral friction velocity (used in laminar
c                               resistance estimates) (m/s)
c               zo:             roughness length (m)
c               rho:            density of the particles (g/m3)
c               temp:           ambient temperature (K)
c               ra:             aerodynamic resistance (s/m)
c               rhoair:         air density (g/m3)
c     Output arguments:
c               vmean:          mass mean deposition velocity (m/s)
c     Routines called:
c               sedrat    stoke     shmidt
c     Called by:
c               surlyr    srlyrf
c
      data visc/1.5e-05/
      dimension rmass(nbin),rsize(nbin)
      dimension vd(20),vg(20)
c
c-----Initialize arrays to zero
c
      xf = 1.0e-06 
      do 5 i = 1,20
        vg(i) = 0.0
        vd(i) = 0.0
    5 continue
c
c-----Estimate the settling velocity
c
      do 10 i = 1,nbin
        d = rsize(i)*xf
        call sedrat(d,rho,rhoair,vsed)
        vg(i) = vsed
   10 continue
c
c-----Estimate the reynolds adjustment parameter
c
      re = ustar*zo/visc
c     b1 = re**0.25
      b1 = 1.0
c
c-----Estimate the stokes law coeff for the critical radius dcrit
c
      dcrit = 3.0e-07
      call sedrat(dcrit,rho,rhoair,vsed)
      call stoke(dcrit,vsed,ustar,st)
      call shmidt(dcrit,temp,sc)
      sc = 1.0/(b1*sc**0.666)
      b2 = sc/st
c
c-----Estimate the schmidt and stokes numbers, and the depostion velocity
c
      do 20 i = 1,nbin
        d = rsize(i)*xf
        vsed = vg(i)
        call stoke(d,vsed,ustar,st)
        call shmidt(d,temp,sc)
        sc = 1.0/(b1*sc**0.666)
        st = b2*st
        eff = 0.25*(sc + st)
        rd = 1.0/(ustar*eff)
        vd(i) = vg(i) + 1.0/(ra + rd + ra*rd*vg(i))
   20 continue
c
c-----Mass weighted mean velocity
c
      sum = 0.0
      do 40 i = 1,nbin
        sum = sum + rmass(i)*vd(i)
   40 continue
      vmean = sum
      return
      end
