c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine partsed(nxx,nyy,nzz,nx,ny,nz,nspec,temper,press,
     &                   ch2o,vg)
c
c-----ATDM v1.00
c
c     Estimates the species-dependent settling velocity for each cell of
c     all grids
c
c-----95/06/05:
c     C Emery SAI
c
c-----95/12/12:
c     -modified code to skip calcs for nh4n and nh4s. (nh4n is set to pno3)
c      nh4s is handled in STEPZ   
c
c     Input arguments:
c               nxx:            max number of grid cells in x-direction     
c               nyy:            max number of grid cells in y-direction
c               nzz:            max number of layers
c               nx:             number of grid cells in x-direction     
c               ny:             number of grid cells in y-direction
c               nz:             number of layers
c               nspec:          number of modeled species
c               temper:         ambient temperature (K)
c               press:          ambient pressure (mb)
c               ch2o:           ambient water vapor (kg/kg)
c     Output arguments:
c               vg:             mass-weighted sedimentation velocity (m/s)
c     Routines called:
c               sedrat
c     Called by:
c               update
c
      include 'parchm.cmd'
      include 'comchem.cmd'
      include 'aerosl.cmd'
      include 'ctlflg.cmd'
c
      dimension temper(nxx,nyy,nzz), press(nxx,nyy,nzz), 
     &          ch2o(nxx,nyy,nzz)
      dimension vg(nxx,nyy,nzz,nspec)
      data eps /0.622/, rd /287./
c
        do 11 k = 1,nz
          do 12 j = 1,ny
            do 13 i = 1,nx
              do 10 l = 1,nspec
                if ( l.eq.knh4n .or. l.eq.knh4s ) goto 10
                if (l.le.ngas .or. .not.lsed) then
                  vg(i,j,k,l) = 0.
                  goto 10
                endif
                rho = 1.e6*rhop(l)
                pa = 100.*press(i,j,k)
                tv = temper(i,j,k)*(eps + ch2o(i,j,k))/
     &               (eps*(1. + ch2o(i,j,k)))
                rhoair = 1000.*pa/(rd*tv)
                isd = iszdist(l)
                sum = 0.
                do 14 n = 1,nbin
                  diam = 1.e-6*rsize(n)
                  call sedrat(diam,rho,rhoair,vsed)
                  sum = sum + rmass(n,isd)*vsed
   14           continue
                vg(i,j,k,l) = sum
   10         continue
	      if ( knh4n .ne. 0 ) vg(i,j,k,knh4n)=vg(i,j,k,kpno3)      
   13       continue
   12     continue
   11   continue
c
      return
      end
