c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
c
      subroutine photol(rk,rkn,klyr,nlayer,zenith,zht,
     &       avterr,cloud,ldark)
c
c     subroutine photol(rk,rkn,klyr,nlayer,zenith,zht,
c    &       avterr,cloud,ldark,llpt)
c
c-----ATDM v1.00
c     96/09/18:dfg
c     -from ATDM v1.00 phot.f (95/10/17)
c     -modified for up to 5 photolysis reactions.
c     -added included files: photrk.cmd,comchem.cmd
c     -added local array rkkk, alfor, fcld for five photolysis reactions.
c     96/10/08:dfg
c     -changed the order of array alfor.  Photolysis rate data are in
c      the order of NO2, HCHO-rad, HCHO-co, O3-O1D, and ALD2.  Previously,
c      O3 was in front HCHO-rad which was not! 
c     -calculated rk in the new order of photolysis data.
c     96/10/22:dfg
c     -add initilizing array fcld (set to 1 for clear sky) right after 
c      j values are loaded.
c     -set go to 60 if klyr=1 instead of return.
c     -set go to 60 if tau.le.5 after loop 50 (instead of return).
c     -set zenith >= 87 to set photolysis rate equal to zero.
c     96/11/20:DFG
c     -removed the fcld calculations which follows method from Chang et al i
c      (1987).
c     -use cloud adjustment factors from UAM-V.  Set cloud cover to be the
c      maximum value from current layer to the top.
c     97/04/23:DFG
c     -change the factor of photolysis rate for H2O2 to 0.189 (previous 0.255
c      which is same as in AOP project.
c     98/01/09:DFG
c     -clean up.
c
c     Determines the NO2, O3, HCHO and ALD2 photolysis rate constants (1/min)
c     as a function of height and solar zenith angle -- adjustments are made
c     for cloud cover following RADM (Chang, et al., JGR Vol. 92, 1987)
c     NO2  -->  NO  + O
c     O3 --> O1D
c     HCHO --> 2 HO2 + CO
c     HCHO --> CO
c     ALD2 -->
c
c     Input arguments:
c               klyr:           current layer index
c               nlayer:         number of layers in column
c               zenith:         solar zenith angle (degrees)
c               zht:            profile of layer interface heights (m AGL)
c               avterr:         terrain elevation (m MSL)
c               cloud:          fractional cloud cover profile (0-1)
c     Output arguments:
c               rk:             reaction rate constants. (1/hr)
c     Called by:
c               rchem2     rchem2f
c
      include 'parchm.cmd'
      include 'photrk.cmd'
c
      logical ldark,llpt
      dimension zht(nlayer), cloud(nlayer)
      dimension fcld(5),alfor(5), rkkk(5),cldadj(11)
      dimension za(10), alt(16),rk(mxgrx),rkn(mxgrx)
      data za /0.,10.,20.,30.,40.,50.,60.,70.,78.,86./
      data alt /0.,.15,.36,.64,.98,.138,.184,.235,.291,3.53,4.21,6.,8.,
     &          10.,12.,14./
      data cwat /1./, rhoh2o /1.e6/, rdrop /1.e-5/
c
c   cloud adjustment factors from RADM (entered, 3/13/95, tcm)
      data cldadj/1.0,0.96,0.92,0.88,0.84,0.80,0.76,0.72
     &               ,0.68,0.64,0.59/
c   cloud adjustment only used if >= 70% cover
c     data cldadj/1.0,1.00,1.00,1.00,1.00,1.00,1.00,0.72
c    &               ,0.68,0.64,0.59/
c
c-----Determine table indices for given zenith and zht
c
      do i=1,5
        rkkk(i)=0.
      end do
c
      ldark=.false.
      if (zenith.ge.87.) then
        rk(1)=0.
        rk(8)=0.
        rk(9)=0.
        rk(16)=0.
        rk(28)=0.
        rk(35)=0.
        rk(43)=0.
        rk(47)=0.
        rk(48)=0.
c        ldark=.true.
        ldark=.false.
c       if (llpt) write(*,*) 'ldark= ', ldark
        return
      endif
c
      do 10 n = 1,9
        if (zenith.ge.za(n) .and. zenith.lt.za(n+1)) then
          zdif1 = abs(zenith-za(n))
          zdif2 = abs(zenith-za(n+1))
          if (zdif1.le.zdif2) then
            iz = n
          else
            iz = n + 1
          endif
          goto 11
        endif
  10  continue
      iz = 10
  11  continue
      zmid = zht(klyr)/2. + avterr
      if (klyr.gt.1) zmid = (zht(klyr) + zht(klyr-1))/2. + avterr
      zmid = zmid/1000.
      do 20 n = 1,15
        if (zmid.ge.alt(n) .and. zmid.lt.alt(n+1)) then
          zdif1 = abs(zmid-alt(n))
          zdif2 = abs(zmid-alt(n+1))
          if (zdif1.le.zdif2) then
            ia = n
          else
            ia = n + 1
          endif
          goto 21
        endif
  20  continue
      ia = 16
  21  continue
c
c-----Load temporary clear sky photolysis rates
c
      do i=1,5
        rkkk(i)=prkn(i,iz,ia)
      end do
c
c-----apply the cloud factor.
c
      do i=1,5
        fcld(i)=1.
      end do
c
      cldmax = 0.
      do 85 kk = klyr,nlayer
        cldmax = amax1(cldmax,cloud(kk))
  85  continue
c
c     icld=int(cloud(klyr)*10.0+1.5)
      icld=int(cldmax*10.0+1.5)
      if (icld.gt.11) icld=11
      if (icld.lt.1) icld=1
c
      do i=1,5
        fcld(i)=cldadj(icld)
      end do
c
c***DFG
c     write(33,34) klyr,icld,cldadj(icld),cloud(klyr)
c34   format('k,icld,cldadj,cloud= ', 2I3,2f6.3)
c
      tmp=rkkk(1)*fcld(1)
      if (tmp.lt.0.0015) then
c        ldark=.true.
        ldark=.false.
        rk(1)=0.
        rk(8)=0.
        rk(9)=0.
        rk(16)=0.
        rk(28)=0.
        rk(35)=0.
        rk(43)=0.
        rk(47)=0.
        rk(48)=0.
        return
      endif
c
c     Calculate adjusted photolysis rate constants.
c     ****  Warning!!!  Micro CB4 hard wired.
c
c---- Rxn 1: NO2 --> NO + O
      rk(1) = rkkk(1)*rkn(1)*fcld(1)
c
c---- Rxn 9: O3 --> O1D
      rk(9) = rkkk(4)*rkn(9)*fcld(4)
c
c---- Rxn 47: CARB --> 2HO2 + CO (use HCHO-rad)
      rk(47) = rkkk(2)*rkn(47)*fcld(2)
c
c---- Rxn 48: CARB --> CO (use HCHO-co)
      rk(48) = rkkk(3)*rkn(48)*fcld(3)
c
c---- Rxn 8: O3 --> O  (ratioed off of NO2)
      rk(8) = 0.053*rk(1)
c
c---- Rxn 16: NO3 -->  (ratioed off of NO2)
      rk(16) = 33.9*rk(1)
c
c---- Rxn 28: HNO2 -->  (ratioed off of NO2)
      rk(28) = 0.1975*rk(1)
c
c---- Rxn 43: H2O2 -->  (ratioed off of HCHO-co)
c     rk(43) = 0.255*rk(48)
      rk(43) = 0.189*rk(48)
c
c     Rxn 35:  PNA (for now, we set it equal to H2O2 photolysis)
      rk(35) = rk(43)
c
c     write(*,*) 'rkkk ', (rkkk(i),i=1,5)
c     call flush(6)
      return
      end
