c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine printr(time,ndate)
c
c-----ATDM v1.00
c
c     Writes average and instantaneous concentrations, and deposited mass
c     to appropriate output files
c
c-----95/03/31:
c     -from UAM-V Revision 1.2  94/02/10  20:59:08  tcm
c     -modifications made following UAM-V to correct vertical nesting code
c-----95/04/05:
c     -added output of dposit and dpositf to deposition files; only those
c      species listed for average output are written to deposition file; 
c      dposit and dpositf reset to zero here
c-----95/07/24:
c     -added wposit and wpositf to output deposition files
c-----95/07/17:
c     -added checks for iudep before writing to deposition file
c-----95/10/26:
c     -added call to new routine FGDEPR.F
c-----99/04/22:DFG
c     -added loops to write oh and o3 coarse concentration
c      when lmoh is true.
c     -added loops to write oh fine grid concentration when
c      lmoh is true and numfin >0.
c     -added ko3 and lfohf in passing arguments in calling FGWRIT.
c      in FGWRIT, fine o3 wil be written out to oh file.
c
c     Input arguments:
c               time:           model time    
c               ndate:          model date
c     Files written:
c               iuavg:          coarse grid average concentration file 
c               51:             coarse grid instantaneous concentration file
c               iudep:          coarse grid deposition file
c               ifavg:          fine grid average concentration file
c               ifinst:         fine grid instantaneous concentration file
c               ifdep:          fine grid deposition file
c     Routines called:
c               fgavwr    fgwrit    fgdepr
c     Called by:
c               atdm
c           
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      include 'com1f.cmd'
      include 'comuam.cmd'
      include 'comchem.cmd'
      include 'com3.cmd'
      real mfac
c
      idex = 31
      jdex = 67
      tmu2 = time/100.
      tmui2 = tmu2 + .001
      tmui1 = tmu2 - .001
      ndu2 = ndate
      tmu1 = tmu2 - dtprnt/100.
      ndu1 = ndate
      if (tmu1 .lt. 0.) then
        tmu1 = tmu1 + 24.
        ndu1 = ndu1 - 1
      endif
      idum = 1
c
      write (*,*) 'start of printr'
c
c   write average concentrations for coarse grid
c
      if (iuavg .gt. 0) then
         do 200 k=1,nospc1
            ksp=jind(k)
c           write(*,*) 'In printr, k, ksp, molwt(ksp): '
c           write(*,*) k, ksp, molwt(ksp)
            do 209 j=2,noy-1
               if (istr(j) .gt. nox) go to 207
               do 211 i=1,istr(j)
                  if(press(i,j,1).le.0. .or. temper(i,j,1).le.0.)then
                     mfac=cpfac
                  else
                     mfac=cpfac*(1013.25/press(i,j,1))*(temper(i,j,1)/273.)
                  endif
                  avcong(i,j,k)=bc(j,4,k,1)*molwt(ksp)/mfac
 211           continue
 207           continue
               if (iend(j) .eq. -9999) go to 219
               do 212 i=iend(j),nox
c                  write (*,*) '212',i,j,k
                  if(press(i,j,1).le.0. .or. temper(i,j,1).le.0.)then
                     mfac=cpfac
                  else
                     mfac=cpfac*(1013.25/press(i,j,1))*(temper(i,j,1)/273.)
                  endif
                  avcong(i,j,k)=bc(j,3,k,1)*molwt(ksp)/mfac
 212           continue
 219           continue
 209        continue
            do 213 i=2,nox-1
               if (jstr(i) .gt. noy) go to 217
               do 214 j=1,jstr(i)
c                  write (*,*) '214', i,j
                  if(press(i,j,1).le.0. .or. temper(i,j,1).le.0.)then
                     mfac=cpfac
                  else
                     mfac=cpfac*(1013.25/press(i,j,1))*(temper(i,j,1)/273.)
                  endif
                  avcong(i,j,k)=bc(i,2,k,1)*molwt(ksp)/mfac
 214           continue
 217           continue
 216           continue
               if (jend(i) .eq. -9999) go to 213
               do 215 j=jend(i),noy
c                  write (*,*) '215', i,j
                  if(press(i,j,1).le.0. .or. temper(i,j,1).le.0.)then
                     mfac=cpfac
                  else
                     mfac=cpfac*(1013.25/press(i,j,1))*(temper(i,j,1)/273.)
                  endif
                  avcong(i,j,k)=bc(i,1,k,1)*molwt(ksp)/mfac
 215           continue
 213        continue
 200     continue
c     write (*,*) 'after 200'
c     
         write(iuavg) ndu1, tmu1, ndu2, tmu2
         do 2 l=1,navspc
            linspc = kavspc(l)
            write(iuavg) idum, (mspec(i,linspc),i=1,10),
     &           ((avcong(i,j,linspc),i=1,nox),j=1,noy)
            write(*,*) 'in printr 2 loop, avcong, linspc, l: '
            write(*,*) avcong(idex,jdex,linspc), linspc, l
 2       continue
      endif
c     write (*,*) 'after 2'
c     
c     write out dump files
c     
      do 100 k=1,nospc1
         do 110 l=1,nlayer
            do 109 j=2,noy-1
               if (istr(j) .gt. nox) go to 411
               do 111 i=1,istr(j)
c                  write (*,*) '111',i,j,k,l
                  conc(i,j,k,l)=bc(j,4,k,l)
 111           continue
 411           continue
               if (iend(j) .eq. -9999) go to 512
               do 112 i=iend(j),nox
c                  write (*,*) '112',i,j,k,l
                  conc(i,j,k,l)=bc(j,3,k,l)
 112           continue
 512           continue
 109        continue
            do 113 i=2,nox-1
               if (jstr(i) .gt. noy) go to 314
               do 114 j=1,jstr(i)
c                  write (*,*) '114',i,j,k,l
                  conc(i,j,k,l)=bc(i,2,k,l)
 114           continue
 314           continue
               if (jend(i) .eq.-9999) go to 613
               do 115 j=jend(i),noy
c                  write (*,*) '115',i,j,k,l
                  conc(i,j,k,l)=bc(i,1,k,l)
 115           continue
 613           continue
 113        continue
 110     continue
 100  continue
c     
c     write (*,*) 'after 100'
c     
      write(*,*) 'writing coarse grid conc data',  ndu2, tmui1, ndu2, tmui2
      write(51) ndu2, tmui1, ndu2, tmui2
      do 4 k=1,nospc1
         do 3 l=1,nlayer
c            write (*,*)'3', k,l
            write(51) idum, (mspec(i,k),i=1,10),
     &           ((conc(i,j,k,l),i=1,nox),j=1,noy)
 3       continue
 4    continue
c     
c     Write out oh and ozone concentration in one file if necessary. 3/23/99, dfg
c     
c      write (*,*) 'lmoh=', lmoh
      if (lmoh) then
         write(iohf) ndu2, tmui1, ndu2, tmui2
         do l=1,nlayer
            write(iohf) idum,(msoh(m),m=1,10),
     &           ((ohcon(i,j,l),i=1,nox),j=1,noy)
         end do
         do l=1,nlayer
            write(iohf) idum,(msho2(m),m=1,10),
     &           ((ho2con(i,j,l),i=1,nox),j=1,noy)
         end do
         do l=1,nlayer
            write(iohf) idum,(msno3(m),m=1,10),
     &           ((no3con(i,j,l),i=1,nox),j=1,noy)
         end do
         do l=1,nlayer
            write(iohf) idum,(msaso2(m),m=1,10),
     &           ((aso2con(i,j,l),i=1,nox),j=1,noy)
         end do
         do l=1,nlayer
            write(iohf) idum,(mspec(i,kh2o2),i=1,10),
     &           ((conc(i,j,kh2o2,l),i=1,nox),j=1,noy)
         end do
         do l=1,nlayer
            write(iohf) idum,(mspec(i,ko3),i=1,10),
     &           ((conc(i,j,ko3,l),i=1,nox),j=1,noy)
         end do
         do l=1,nlayer
            write(iohf) idum,(mspec(i,kpec),i=1,10),
     &           ((conc(i,j,kpec,l),i=1,nox),j=1,noy)
         end do
      endif
c     
c     write out deposition file
c     
      if (iudep .gt. 0) then
         write(iudep) ndu1, tmu1, ndu2, tmu2
         do 10 l=1,navspc
            linspc = kavspc(l)
            write(iudep) idum, (mspec(i,linspc),i=1,10),
     &           ((dposit(i,j,linspc),i=1,nox),j=1,noy)
            write(iudep) idum, (mspec(i,linspc),i=1,10),
     &           ((wposit(i,j,linspc),i=1,nox),j=1,noy)
 10      continue
         do 13 l = 1,nospc1
            do 12 j = 1,noy
               do 11 i = 1,nox
                  dposit(i,j,l) = 0.
                  wposit(i,j,l) = 0.
 11            continue
 12         continue
 13      continue
c         write (*,*) 'after 13'
      endif
c     
c     write out fine grid inst and avg concs, and deposited mass
c     
      if (ifavg .gt. 0) write(ifavg) time, ndate
      write (*,*) 'writing fine grid conc', time, ndate
      if (ifinst .gt. 0) write(ifinst) time, ndate
      if (ifdep .gt. 0) write(ifdep) time, ndate
      if (lmoh .and. ifohf.gt.0) write(ifohf) time,ndate
      do 5 ifine=1,numfin
         if (ifavg .gt. 0) then
            call fgavwr(ifavg,nfx(ifine),nfy(ifine),mxspec,
     &           navspc,kavspc,avconf(1,1,ifine))
         endif
c     
c     Write out oh concentration if necessary. 4/14/99, dfg
c     
         if (lmoh) then
            call wrfo3oh(ifohf,nfx(ifine),nfy(ifine),nfz(ifine),
     &                   ohconf(1,1,1,ifine),no3conf(1,1,1,ifine),
     &                   ho2conf(1,1,1,ifine),aso2conf(1,1,1,ifine))
c
c           do k=1,nfz(ifine)
c              write(ifohf) ((ohconf(i,j,k,ifine),i=1,nfx(ifine)),
c    &              j=1,nfy(ifine))
c           end do
c           do k=1,nfz(ifine)
c              write(ifohf) ((ho2conf(i,j,k,ifine),i=1,nfx(ifine)),
c    &              j=1,nfy(ifine))
c           end do
c           do k=1,nfz(ifine)
c              write(ifohf) ((no3conf(i,j,k,ifine),i=1,nfx(ifine)),
c    &              j=1,nfy(ifine))
c           end do
c           do k=1,nfz(ifine)
c              write(ifohf) ((aso2conf(i,j,k,ifine),i=1,nfx(ifine)),
c    &              j=1,nfy(ifine))
c           end do
         endif
c
c         write (*,*) 'call fgwrit'
         if (ifinst .gt. 0) then
            call fgwrit(ifinst,nfx(ifine),nfy(ifine),nfz(ifine),nospec,
     &           cncf(1,1,1,ifine),ifohf,ko3,kh2o2,kpec)
         endif
c         write (*,*) 'call fgdepr'
         if (ifdep .gt. 0) then
            call fgdepr(ifdep,nfx(ifine),nfy(ifine),mxspec,
     &           navspc,kavspc,dpositf(1,1,ifine),
     &           wpositf(1,1,ifine))
            do 20 l = 1,mxspec
               do 21 i = 1,mxfxby
                  dpositf(i,l,ifine) = 0.
                  wpositf(i,l,ifine) = 0.
 21            continue
 20         continue
         endif
 5    continue
c      write (*,*) 'after 5'
c     
c     flush causes buffer to be flushed so that data is safely
c     written to disk
c     
      if (iuavg .gt. 0) call flush(iuavg)
      call flush(51)
      if (lmoh) call flush(iohf)
      if (lmoh .and. ifohf.gt.0) call flush(ifohf)
      if (iudep .gt. 0) call flush(iudep)
      if (ifavg .gt. 0) call flush(ifavg)
      if (ifinst .gt. 0) call flush(ifinst)
      if (ifdep .gt. 0) call flush(ifdep)
c     
      write (*,*) 'end of subroutine'
c
      return
      end
