c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
c
      subroutine setup(time,ndate,tend,idaynf,totime,tdatnx,
     &                 lrstrt,idebug)
c
c-----ATDM v1.00
c
c     Sets up for model run
c
c-----95/02/22:
c     -from UAM-V Revision 1.4  94/10/18  17:48:15  tcm
c     -removed inputs and code related to running hourly O3 averages
c     -removed inputs and code related to PiG
c     -removed inputs and code related to chemistry aggregation
c     -reinstated read of particulate size distributions
c     -removed inputs and code related to linear chemistry
c     -removed call to TIRATE to setup photalysis lookup table
c     -removed inputs and code related to non-gridded receptors
c     -removed inputs and code related to the albedo/haze/ozone file
c     -modified summary output for the above changes
c     -major modifications for ATDM in sigma coords
c-----95/03/31 (v0.02):
c     -modifications made following UAM-V to correct vertical nesting code
c-----95/08/24:
c     -Improved several diagnostic output formats
c-----96/9/10:dfg
c     -added an option for chemical mechanism
c     -added code that related to the chemical mechanism options.
c     -added call to tirate for calculating rate constants when chemical 
c      mechanism option is true.
c     -added included 'photrk.cmd'
c     -added read statement for iteration and minimum time step for step4
c      before the number of fine grid is read.
c     -added open unit for photolysis data and read the data when Micro
c      CB4 is specified. (right after open oh tables).
c-----99/4/14 : DFG
c     -added initilizing radical array for new chemistry solver to use.
c-----99/4/22:DFG
c     -added data statement for msoh as ('O   ','H   ',8*'    ')
c     -added option to write out OH & O3 in a separate file.
c     -added a check when reading modeled species.  If the last species
c      is OH, then set LMOH is true.
c     -added to read oh_o3 coarse and fine grid instantaneous file units
c      file names.
c     -added statements for writting headers for coarse oh_o3 file if
c      LMOH is true.
c     -added another flag LO3 for user to provide after 'lstagr' is read.
c     -added to read one oh_o3 fine grid file is LO3 is true and numfin >0.
c      users do not need to give oh tables. The units for OH table can be set
c      to zero.
c     -added call to HEDO3 if LO3 is true.
c     -changed passing argument to EFTOT, i.e. passing nfemit(ifine)
c      (actual # of of emitted species) instead of passing nospec
c      (# of modeled species)
c
c     Input arguments:
c               idebug:         diagnostic debug flag
c     Output arguments:
c               time:           model start hour
c               ndate:          model start date
c               tend:           model end hour
c               idaynf:         model end date
c               totime:         total simulated time (0) 
c               tdatnx:         time at which to read new inputs 
c               lrstrt:         model restart flag
c     Files read:
c               7:              ATDM control file
c               isurf:          surface cover type
c               icbc:           initial concentration and boundary 
c                               conditions (this variable is reused
c                               for each of 2 files:  initial conditions
c                               and lateral boundaries)
c     Files written:
c               6:              standard output file unit (*)
c               8:              diagnostic file
c               66:             output message file
c               51:             instantaneous concentration output file
c               iuavg:          average concentration output file
c               iudep:          deposition output file
c     Routines called:
c               lftpak    maputg    chread    rvmmap    chksig    rdfsrf
c               flxnit    hedpts    gunit     hedqar    getqar    getpt
c               eftot     grdsp     hedibc    getic     heddv     unchek
c               bnddef    restrt    diagn     update 
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'deffin.cmd'
      include 'shrfin.cmd'
      include 'com1f.cmd'
      include 'comuam.cmd'
      include 'aerosl.cmd'
      include 'oneday.cmd'
      include 'latlon.cmd'
      include 'comchem.cmd'
      include 'com3.cmd'
      include 'efcon.cmd'
      include 'ibccon.cmd'
      include 'hedmsg.cmd'
      include 'advec.cmd'
      include 'iemcon.cmd'
      include 'ctlflg.cmd'
      include 'stack.cmd'
      include 'ohtbl.cmd'
      include 'photrk.cmd'
      include 'kweight.cmd'
c
      logical lerr
      dimension vcheck(5)
      integer aem
      logical lrstrt,lsuprs,liccon
      character*140 fn, fn2
      character*10 mdum
      character*10 nams
      character*10 ifile
      character*10 ifili
      character*10 ifild
      character*1 mblank
      integer utmzon
c
c-----Logical flag lsuprs suppress point source info on diag output file
c
      data lsuprs/.true./
      data liccon/.false./
c     data cloud /maxagg*0.0/, rain/mxset*0.0/
c     data cloudf /mxaggf*0.0/, rainf/mxsetf*0.0/
      data ifile /'AVERAGE   '/, ndum /1/
      data ifili /'INSTANT   '/
      data ifild /'DEPOSITION'/
      data mblank /' '/
      data izero /0/, fzero /0./
      data msoh/'O   ','H   ',8*'    '/
      data msho2/'H   ','O   ','2   ',7*'    '/
      data msno3/'N   ','O   ','3   ',7*'    '/
      data msaso2/'A   ','S   ','O   ','2   ',6*'    '/
c
c-----Initialize rain and cloud arrays 
c

      do j=1,mxy
	do i=1,mxx
	  rain(i,j)=0.0
	  do k=1,mxz
	    cloud(i,j,k)=0.0
	  enddo
	enddo
      enddo
      do i=1,mxfxby
	do j=1,maxfin
	  rainf(i,j)=0.0
	  do k=1,mxfz
	    cloudf(i,k,j)=0.0
	  enddo
	enddo
      enddo
c
c  check vertical parameter definition (mxfz must be => mxz)
c
      if (mxfz .lt. mxz) then
        write(*,*) 'error: parameter mxfz must be >= mxz'
        write(*,*) 'Increase mxfz and recompile all subroutines.'
        stop
      endif
c
c-----Bit six of idebug on (i.e., the thirty-twos location) will turn
c     on debug writes in setup.
c
      idiag = and(idebug,32)
c
c-----Open output files names from control input file (unit 7)
c
      read(7,7001) iout,fn
      if (iout .ne. 66) then
        write(*,*) 'error: out file must be opened on unit 66'
        stop
      endif
      open(66,file=fn,form='formatted')
      read(7,7001) iout,fn
      if (iout .ne. 8) then
        write(*,*) 'error: diag file must be opened on unit 8'
        stop
      endif
      open(8,file=fn,form='formatted')
      read(7,7001) iuavg,fn
      if (iuavg .gt. 0) then
        open(iuavg,file=fn,form='unformatted')
        write(8,7020) iuavg, fn
 7020   format(' open unit=',i2,'  file=',a,' for average conc output')
        read(7,7025) navspc, (mavspc(l),l=1,6)
        if (navspc.gt.6) read(7,70251) (mavspc(l),l=7,navspc)
 7025   format(i2,6a10)
70251   format(2x,6a10)
c
c---- Check OH species for print out option, dfg, 4/14/99.
c
        iohf=0
        lmoh=.false.
        if (mavspc(navspc).eq.'OH        ') then
          navspc=navspc-1
          lmoh=.true.
          read(7,7001) iohf,fn
          open(iohf,file=fn,form='unformatted')
        endif
c**
        do 15 l=1,navspc
          call lftpak(mavspc(l))
   15   continue
      else
        write(8,'(a)') ' no file opened for average conc output'
      endif
      read(7,7001) iudep,fn
      if (iudep .gt. 0) then
        open(iudep,file=fn,form='unformatted')
        write(8,7022) iudep,fn
 7022   format(' open unit=',i2,'  file=',a,' for deposition output')
      else
        write(8,'(a)') ' no file opened for deposition output'
      endif
      read(7,7001) iout,fn
      if (iout .ne. 51) then
        write(*,*) 'error: inst conc file must be opened on unit 51'
        stop
      endif
      open(51,file=fn,form='unformatted')
      write(8,7021) fn
 7021 format(' open unit=51  file=',a,' for dump (inst) conc out')
c
c-----Open fine grid output concentration files (standard units are
c     62 for average concs, 52 for inst concs)
c
      read(7,7001) ifavg,fn
      if (ifavg .gt. 0) then
        open(ifavg,file=fn,form='unformatted')
        write(8,7030) ifavg, fn
 7030   format(' open unit=',i2,' file=',a,' for fine grid ',
     &         'average conc output')
      else
        write(8,7031) 
 7031   format(' no file opened for fine grid ',
     &         'average conc output')
      endif
      read(7,7001) ifdep,fn
      if (ifdep .gt. 0) then
        open(ifdep,file=fn,form='unformatted')
        write(8,7032) ifdep,fn
 7032   format(' open unit=',i2,' file=',a,' for fine grid ',
     &         'deposition output')
      else
        write(8,7033) 
 7033   format(' no file opened for fine grid ',
     &         'deposition output')
      endif
      read(7,7001) ifinst,fn
      if (ifinst .gt. 0) then
        open(ifinst,file=fn,form='unformatted')
        write(8,7035) ifinst, fn
 7035   format(' open unit=',i2,'  file=',a,' for fine grid ',
     &         'inst conc output')
      else
        write(8,7036) 
 7036   format(' no file opened for fine grid ',
     &         'inst conc output')
      endif
c
c
c --- open fine grid output file for OH and Ozone if necessary.  dfg, 4/14/99
c
      ifohf=0
      if (lmoh) then
        read(7,7001) ifohf,fn
        if (ifohf .gt. 0) then
          open(ifohf,file=fn,form='unformatted')
        endif
      endif
c
c-----Read run control information 
c
      if (idiag .ne. 0) then
        write(*,3001) time,ndate
 3001   format(' enter setup  ',f10.1,i10)
      endif
c
c     Run message
c
      read(7,10) msg
   10 format(a80)
      write(8,20) msg
      write(66,20) msg
   20 format(//,1x,a80//)
c
c     Run flags
c
c*** Add one more flag for mechanism option.
c    lmech = T -- use Micro CB4 mechanism.
c    lmech = F -- use parameterized chemistry.
c*** Add another flag for using ozone and OH from REMSAD instantaneous files
c    lo3 = T -- need ozone and OH file
c    lo3 = F -- do not need ozone and OH file
c    if lo3 is true, lmoh and lmech must be false!!!
c
      read(7,23) lrstrt,ldepn,lwet,lsed,lpmchm,ltxchm,lmech
      read(7,23) lcart,larea,lpts,l1eday,l1day,lstagr,lo3
      if (lmech .and. lo3) then
        write(*,*) 'Error -- lmech and lo3 can not be true',
     &        'at the same time'
        stop
      endif
      if (lmoh .and. lo3) then
        write(*,*) 'Error -- lmoh and lo3 can not be true',
     &        'at the same time'
        stop
      endif
   23 format(7l10)
c
c     Time/Date parameters
c
      read(7,30) tstr,idystr,tend,idaynf,dtdata,dtprnt,dtstep
      read(7,31) dlong,dlat,nox,noy,nlayer,nospec
      datadt=0.6*dtdata
   30 format(f10.0,i10,f10.0,i10,3f10.0)
   31 format(2f10.0,4i10)
      nospc1=nospec
c
c     Grid parameters
c
      read(7,50) wlong,elong,slat,nlat,tz,utmzon
      read(7,118) xorg,yorg
   50 format(4f10.0,f10.0,i10)
  118 format(2f10.0)
c
c     Make sure nox=mxx and noy=mxy otherwise errors will result
c
      if (nox.gt.mxx .or. noy.gt.mxy) then
        write(8,33) nox,noy
        write(66,33) nox,noy
        write(*,33) nox,noy
   33   format('   error parameters mxx,mxy not consistent with input'
     &  ,' nox,noy ',2i5)
        stop1
      endif
c
c     Make sure nlayer < mxz otherwise errors will result
c
      if (nlayer.gt.mxz) then
        write(8,34) mxz, nlayer
        write(66,34) mxz, nlayer
        write(*,34) mxz, nlayer
   34   format('   error parameter mxz not consistent with input'
     &  ,' nlayer ',2i5)
        stop2
      endif
c
c     Coarse grid vertical structure (sigma coordinates)
c
      sigma(1) = 1.
      read(7,52) pmodtop,(sigma(k+1),k=1,nlayer)
   52 format(21f7.4)
c
c     Fine grid definitions
c
      read(7,'(i10)') numfin
      if (numfin .gt. maxfin) then
        write(8,'(a,i4,/,a,i4)')
     &           ' Max number of fine grids exceeded: ',numfin,
     &           ' Max is ',maxfin
        write(66,'(a,i4,/,a,i4)')
     &           ' Max number of fine grids exceeded: ',numfin,
     &           ' Max is ',maxfin
        stop3
      endif
      do 35 ifine=1,numfin
        read(7,'(6i10)') ixfb(ifine), jyfb(ifine), 
     &                   ixfe(ifine), jyfe(ifine), 
     &                   nhf(ifine), nfz(ifine)
        if (nfz(ifine).gt.mxfz) then
          write(8,36) mxz, nfz(ifine)
          write(66,36) mxz, nfz(ifine)
          write(*,36) mxz, nfz(ifine)
   36     format('   error parameter mxfz not consistent with input'
     &    ,' nfz ',2i5)
          stop4
        endif
        sigmaf(1,ifine) = 1.
        read(7,53) (sigmaf(k+1,ifine),k=1,nfz(ifine))
   53 format(6x,20f7.4)
   35 continue
c
c-----Echo input parameters 
c
      write(8,40) tstr,idystr,tend,idaynf,dtdata,dtprnt,dtstep,
     & dlong,dlat,nox,noy,nlayer,nospec
      write(66,40) tstr,idystr,tend,idaynf,dtdata,dtprnt,dtstep,
     & dlong,dlat,nox,noy,nlayer,nospec
   40 format(/,2x,'Simulation start time/date:    ',f10.0,2x,i10,/,
     &       2x,'Simulation end time/date:      ',f10.0,2x,i10,/,
     &       2x,'Data input interval:                     ',f10.0,/,
     &       2x,'Output interval:                         ',f10.0,/,
     &       2x,'Maximum coarse grid time step (min):     ',f10.0,//,
     &       2x,'Longitude grid spacing:                  ',f10.3,/,
     &       2x,'Latitude grid spacing:                   ',f10.3,/,
     &       2x,'Grid dimensions (x-, y-, z-cells):  ',3i5,/,
     &       2x,'Number of species:                       ',i10,/)
      write(8,55) wlong,elong,slat,nlat,tz
      write(66,55) wlong,elong,slat,nlat,tz
   55 format(2x,'Longitude of western boundary:           ',f8.2,/,
     &       2x,'Longitude of eastern boundary:           ',f8.2,/,
     &       2x,'Latitude of southern boundary:           ',f8.2,/,
     &       2x,'Latitude of northern boundary:           ',f8.2,/,
     &       2x,'time zone:                               ',f4.1)
      if (lcart) then
        write(8,56) xorg,yorg,utmzon
        write(66,56) xorg,yorg,utmzon
      endif
   56 format(2x,'UTM x-origin (km):                       ',f8.2,/,
     &       2x,'UTM y-origin (km):                       ',f8.2,/,
     &       2x,'UTM zone:                                ',f4.1)
      write(8,58) pmodtop
      write(66,58) pmodtop
      write(8,580) sigma(1)
      write(66,580) sigma(1)
      do 558 k = 2,nlayer+1
        write(8,581) k-1,sigma(k)
        write(66,581) k-1,sigma(k)
  558 continue
   58 format(2x,'Model top pressure (mb):                 ',f8.2)
  580 format(2x,'Surface:      ',2x,'   Sigma: ',f6.3) 
  581 format(2x,'Top of Layer: ',i2,'   Sigma: ',f6.3)
      do 60 ifine = 1,numfin
        write(8,59) ifine
        write(66,59) ifine
        write(8,580) sigmaf(1,ifine)
        write(66,580) sigmaf(1,ifine)
        do 559 k = 2,nfz(ifine)+1
          write(8,581) k-1,sigmaf(k,ifine)
          write(66,581) k-1,sigmaf(k,ifine)
  559   continue
   59   format(2x,'Fine Grid #: ',i3)
   60 continue
      write(8,26) lrstrt,ldepn,lwet,lsed,lpmchm,ltxchm,lmech,
     &            lcart,larea,lpts,l1eday,l1day,lstagr,lo3
      write(66,26) lrstrt,ldepn,lwet,lsed,lpmchm,ltxchm,lmech,
     &            lcart,larea,lpts,l1eday,l1day,lstagr,lo3
   26 format('        restart option = ',l2/,
     &       ' dry deposition option = ',l2/,
     &       ' wet deposition option = ',l2/,
     &       '       sediment option = ',l2/,
     &       '   PM chemistry option = ',l2/,
     &       '  TOX chemistry option = ',l2/,
     &       '  CB4 Mechanism option = ',l2/,
     &       '      cartesian option = ',l2/,
     &       '    area source option = ',l2/,
     &       '   point source option = ',l2/,
     &       '    1 day emiss option = ',l2/,
     &       '  1 day met inp option = ',l2/,
     &       ' staggered wind option = ',l2/,
     &       ' O3 and OH file option = ',l2)
      if (.not.lcart) write(8,168)
      if (lcart) write(8,170)
  168 format('  long/latitude coordinate system')
  170 format('  cartesian coordinate system; dlong/dlat are in km')
c
c***
c-----Read iteration step and minimum time step to be used in step4.
c
      read(7,73) dtmin4,maxitr,rerror
      write(66,74) dtmin4,maxitr,rerror
   73 format(f8.0,i10,f10.0)
   74 format(' dtmin1= ',f10.6,' maxitr= ',i5,' rerror= ',f10.6)
c***
c
c-----Read in unit numbers and file names for input and output
c
      read(7,7001) ichem,fn
 7001 format(i2,a)
      open(ichem,file=fn,form='formatted',status='old')
      write(8,7002) ichem,fn
 7002 format(' open unit=',i2,'  file=',a,' for chem file')
c
      read(7,7001) ioh1,fn
c***
      if (lmech) then
        write(*,*) 'Micro CB4 mechanism has been specified; ',
     &             'you do not need lower OH lookup table.'
      endif
c***
      if (ioh1.gt.0) then
        open(ioh1,file=fn,form='formatted',status='old')
        write(8,7018) ioh1,fn
 7018   format(' open unit=',i2,'  file=',a,' for lower OH file')
      else
        if (lmech) go to 70181
        if (lpmchm) then
          if (.not.lo3) then
            write(*,*)'PM Chemistry has been specified; you must',
     &                ' provide 2 OH lookup tables!'
            stop
          endif
        endif
        if (ltxchm) then
          if (.not.lo3) then
            write(*,*)'Tox Chemistry has been specified; you may need',
     &                ' to provide 2 OH lookup tables!'
          endif
        endif
      endif
70181 continue
c
      read(7,7001) ioh2,fn
c***
      if (lmech) then
        write(*,*) 'Micro CB4 mechanism has been specified; ',
     &             'you do not need upper OH lookup tables.'
      endif
c***
      if (ioh2.gt.0) then
        open(ioh2,file=fn,form='formatted',status='old')
        write(8,7019) ioh2,fn
 7019   format(' open unit=',i2,'  file=',a,' for upper OH file')
      else
        if (lmech) go to 70191
        if (lpmchm) then
          if (.not.lo3) then
            write(*,*)'PM Chemistry has been specified; you must',
     &                ' provide 2 OH lookup tables!'
            stop
          endif
        endif
        if (ltxchm) then
          if (.not.lo3) then
            write(*,*)'Tox Chemistry has been specified; you may need',
     &                ' to provide 2 OH lookup tables!'
          endif
        endif
      endif
70191 continue
c***
      read(7,7001) iphot,fn
      
      if (lmech) then
        write(*,*) 'Micro CB4 mechanism has been specified; ',
     &             'you need a look-up table for photolysis rate'
      endif
c
      if (iphot.gt.0) then
         open(iphot,file=fn,form='formatted',status='old')
         write(8,70031) iphot,fn
70031 format(' open unit=',i2,'  file=',a,' for photolysis data')
c
c   If necessary, read photolysis rates as function of height, 
c   zenith angle under constant albedo, ozone column and haze for 
c   5 photolysis reactions: NO2, O3, HCHO-rad, HCHO-co and ALD2.
c
c    iz=16 -- 16 different elvations.
c    jz=10 -- 10 different zenith angle.
c    iphk=5 -- 5 photolysis reactions.
c
c-----Read in photolysis rates. 
c
         read(iphot,*)
         do 130 iz=1,16
           read(iphot,*)
           do 120 iphk=1,5
             read(iphot,131) (prkn(iphk,jz,iz),jz=1,10)
 120       continue
 130     continue
 131     format(10E12.3)
      else
        if (.not.lmech) go to 70032
        write(*,*) 'Micro CB4 mechanism has been specified; ',
     &             'you must provide a file for photolysis rate.'
        stop
      endif
70032 continue
c
      read(7,7001) iwind,fn
      open(iwind,file=fn,form='unformatted',status='old')
      write(8,7003) iwind,fn
 7003 format(' open unit=',i2,'  file=',a,' for wind file')
      read(7,7001) iprs,fn
      open(iprs,file=fn,form='unformatted',status='old')
      write(8,7004) iprs,fn
 7004 format(' open unit=',i2,'  file=',a,' for pressure file')
      read(7,7001) itemp,fn
      open(itemp,file=fn,form='unformatted',status='old')
      write(8,7005) itemp,fn
 7005 format(' open unit=',i2,'  file=',a,' for temper file')
      read(7,7001) ih2o,fn
      open(ih2o,file=fn,form='unformatted',status='old')
      write(8,7006) ih2o,fn
 7006 format(' open unit=',i2,'  file=',a,' for water file')
      read(7,7001) iclh2o,fn
      open(iclh2o,file=fn,form='unformatted',status='old')
      write(8,7009) iclh2o,fn
c      write (*,*) 'in setup and cloud', iclh2o, fn
 7009 format(' open unit=',i2,'  file=',a,' for cloud water file')
      read(7,7001) irnh2o,fn
      open(irnh2o,file=fn,form='unformatted',status='old')
      write(8,7119) irnh2o,fn
 7119 format(' open unit=',i2,'  file=',a,' for rain water file')
      read(7,7001) isurf,fn
      open(isurf,file=fn,form='unformatted',status='old')
      write(8,7007) isurf,fn
 7007 format(' open unit=',i2,'  file=',a,' for surf file')
      read(7,7001) icl2,fn
      open(icl2,file=fn,form='formatted',status='old')
      write(8,70071) icl2,fn
70071 format(' open unit=',i2,'  file=',a,' for cl2 file')
      read(7,7001) iterr,fn
      if (iterr.gt.0) then
        open(iterr,file=fn,form='unformatted',status='old') 
        write(8,7016) iterr,fn
 7016   format(' open unit=',i2,'  file=',a,' for terrain file')
      else
        if (lpmchm .or. ltxchm) then
          write(*,*)'Chemistry has been specified; you must provide',
     &              ' a gridded terrain input file!'
          stop
        endif
      endif
      if (lcart .and. utmzon .eq. 0) then
        read(7,7001) icoord,fn
        if (icoord.gt.0) then
          open(icoord,file=fn,form='formatted',status='old') 
          write(8,'(a,i2,a,a,a)') ' open unit=',icoord, 
     &          '  file=', fn,' for terrain file'
        else
          write(*,*)'Non-UTM cartesian coordinates systems must',
     &                ' provide cell lat/lon on coord file!'
          stop
        endif
      endif
      read(7,7001) edf,fn
      if (edf.gt.0) then
        open(edf,file=fn,form='unformatted',status='old')
        write(8,7008) edf,fn
 7008   format(' open unit=',i2,'  file=',a,' for pt ems file')
      else
        if (lpts) then
          write(*,*)'Treatment of point sources has been specified'
          write(*,*)'you must provide a point source emissions file'
          stop
        endif
      endif
      read(7,7001) aem1,fn
      if (aem1.gt.0) then
        open(aem1,file=fn,form='unformatted',status='old')
        write(8,7010) aem1,fn
 7010   format(' open unit=',i2,'  file=',a,' for aem weekday file')
      else
        if (larea) then
          write(*,*)'Treatment of area sources has been specified'
          write(*,*)'you must provide an area source emissions file'
          stop
        endif
      endif
      read(7,7001) aem2,fn
      if (aem2.gt.0) then
        open(aem2,file=fn,form='unformatted',status='old')
        write(8,7011) aem2,fn
 7011   format(' open unit=',i2,'  file=',a,' for aem saturday file')
      else
        aem2=aem1
      endif
      read(7,7001) aem3,fn
      if (aem3.gt.0) then
        open(aem3,file=fn,form='unformatted',status='old')
        write(8,7012) aem3,fn
 7012   format(' open unit=',i2,'  file=',a,'for aem sunday file')
      else
        aem3=aem1
      endif
      read(7,7001) icbc,fn
      if (icbc .gt. 0) then
        open(icbc,file=fn,form='unformatted',status='old')
        write(8,7013) icbc,fn
 7013   format(' open unit=',i2,'  file=',a,'for initial conditions')
      endif
      read(7,7001) io3,fn
c***
      if (lmech) then
        write(*,*) 'Micro CB4 mechanism has been specified; ',
     &             'you do not need O3 file.'
      endif
c***
      if (io3.gt.0) then
        open(io3,file=fn,form='unformatted',status='old') 
        write(8,7014) io3,fn
 7014   format(' open unit=',i2,'  file=',a,' for ozone file')
      else
        if (lmech) go to 70141
        if (lpmchm) then
          write(*,*)'PM Chemistry has been specified; you must provide',
     &              ' a gridded ozone input file!'
          stop
        endif
        if (ltxchm) then
          write(*,*)'Tox Chemistry has been specified; you may need',
     &              ' to provide a gridded ozone input file!'
        endif
      endif
c
c*** added if lo3=true and numfin > 0, then you need to
c    provide O3 OH NO3 H2O2 HO2 fine grid files. dfg ,4/15/99
c
      io3f=0
      if (lo3 .and. numfin.gt.0) then
        read(7,7001) io3f,fn
        if (io3f.gt.0) then
          open(io3f,file=fn,form='unformatted',status='old')
          write(8,7014) io3f,fn
        else
          write(*,*) 'Error: you have set lo3=true & with nested',
     &                 'grids, you need fine grid file for O3 OH NO3 H2O2 HO2!'
        endif
      endif
70141  continue
c
      read(7,7001) irain,fn
      if (irain.gt.0) then
        open(irain,file=fn,form='unformatted',status='old')
        write(8,7015) irain,fn
 7015   format(' open unit=',i2,'  file=',a,' for rain file')
      else
        if (lwet) then
          write(*,*)'Wet deposition has been specified; you must',
     &              ' provide a gridded rain input file!'
          stop
        endif
      endif
      read(7,7001) ikv,fn
      open(ikv,file=fn,form='unformatted',status='old')
      write(8,7017) ikv,fn
 7017 format(' open unit=',i2,'  file=',a,' for kv dif file')
c
c-----Calculate lat/lon of cell centroids
c
c     (UTMZON of 0 with lcart true will be used to denote
c      rectangular coord. sys. in km, other than UTM.
c      Lat/lon of center of each coarse grid cell must be
c      provided on a coordinate file.
c      When reading coord file, check that all cells are 
c      defined and do gross checks on reasonableness of 
c      lon/lat values.
c      mod -- tcm, 9/21/01)
c
      do 189 j=1,noy
        do 186 i=1,nox
          if (lcart) then
            if (utmzon .eq. 0) then
              read(icoord,*) iic, jjc, cn, ct
              if (iic .ne. i .or. jjc .ne. j) then
                write(*,*) 'lon/lat out of sequence.'
                write(*,*) 'i,j,iic,jjc: ',i,j,iic,jjc
                stop
              endif
              if (cn .gt. 180. .or. cn .lt. -180. .or.
     &            ct .gt. 90. .or. ct .lt. -90.) then
                write(*,*) 'Invalid Lon or Lat value.'
                write(*,*) 'iic,jjc,ctlon,ctlat: ',iic,jjc,cn,ct
                stop
              endif
              ctlon(i,j) = cn
              ctlat(i,j) = ct
            else
              xloc = xorg + (float(i) - 0.5)*dlong
              yloc = yorg + (float(j) - 0.5)*dlat
              call maputg(xloc,yloc,utmzon,alon,alat)
              ctlon(i,j) = alon
              ctlat(i,j) = alat 
            endif
          else
            if (utmzon.ge.0) then
              write(*,*)'You have specified Lat/Lon coordinates'
              write(*,*)'UTMZON must be set negative!'
              stop
            endif
            ctlon(i,j) = wlong + (float(i) - 0.5)*dlong
            ctlat(i,j) = slat + (float(j) - 0.5)*dlat
          endif
  186   continue
  189 continue
      if (lcart .and. utmzon .eq. 0) close(icoord)
c
c-----Input chemical rate constants
c
      if (idiag.ne.0) write(8,*)'entering chread...'
      write (*,*) 'before chread'
      call chread(ichem,nn1,nnhg,idebug)
      write (*,*) 'after chread'
      if(idiag.ne.0)write(8,*)'returned from chread...'
      if (nn1 .ne. nospec .or. nn1.gt.mxspec) then
         write(*,*) 'inconsistent number of species'
         write(*,*) 'Chemparam file: ',nn1
         write(*,*) 'ATDM Control file: ',nospec
         write(*,*) 'Max number of species: ',mxspec
         stop
      endif
c***
c-----Calculate the look-up table for the temperature dependent
c     rate constants if Micro CB4 mechanism has been specified.
c
      if (lmech) then
        call tirate
        go to 191
      endif
c***
c
c-----Read lower and upper OH lookup tables
c
      if (ioh1.ne.0 .and. ioh2.ne.0) then
        read(ioh1,*)
        read(ioh1,*)
        read(ioh1,190) (oh01(i),i=1,nmoh)
        read(ioh2,*)
        read(ioh2,*)
        read(ioh2,190) (oh02(i),i=1,nmoh)
      endif
 190  format(7(f6.4,1x))
 191  continue
c
c-----Read in fractional land use for each cell
c
      write(*,3024) nox,noy,mlus
 3024 format(' read surf file',3i10)
      read(isurf) (((fland(i,j,k),i=1,nox),j=1,noy),k=1,mlus)
      close(isurf)
c
c-----Read chlorine concentrations
  192 read(icl2,*,end=193) ii,jj,cc
      if (ii .ge. 1 .and. ii .le. nox .and.
     &    jj .ge. 1 .and. jj .le. noy) then
        cl2con(ii,jj) = cc*1.e-6
      else
        write(*,*) 'Bad cell in chlorine file.'
      endif
      go to 192
  193 close(icl2)
c
c-----Read in mean terrain elevations for each cell
c
      if (iterr.ne.0) then
        write(*,3025) nox,noy
 3025   format(' read terrain file',2i10)
        read(iterr) ((avterr(i,j),i=1,nox),j=1,noy)
        close(iterr)
      endif
c
c-----Set up nested grid parameters
c
c     call rvmmap(nox,noy,nlayer,8,tstr,idystr,idebug)
      call rvmmap(nox,noy,8,idebug)
c
c-----Check relationships between coarse and fine grid layer structures
c
      do 720 ifine=1,numfin
        if (ifgptr(ifine) .eq. 0) then
          call chksig(ifgptr(ifine),nlayer,sigma,ifine,nfz(ifine),
     &               sigmaf(1,ifine))
        else
          ifget = ifgptr(ifine)
          call chksig(ifget,nfz(ifget),sigmaf(1,ifget),ifine,nfz(ifine),
     &               sigmaf(1,ifine))
        endif
  720 continue
c
c-----Optionally open wind files for any fine grid
c
      do 4560 ifine=1,numfin
        read(7,7001) ifwind(ifine),fn
        if (ifwind(ifine) .gt. 0) then
          open(ifwind(ifine),file=fn,form='unformatted',status='old')
          write(8,'(a,i2,a,a,/,a,i2)') ' open unit ',ifwind(ifine),
     &         ' file ',fn,' for wind for fine grid ',
     &         ifine
        else
          write(8,'(a,a,i2)') ' no file opened for wind ',
     &         'for fine grid ',ifine
        endif
 4560 continue
c
c-----Optionally open vert diffusion files for any fine grid
c
      do 4570 ifine=1,numfin
        read(7,7001) ifkv(ifine),fn
        if (ifkv(ifine) .gt. 0) then
          open(ifkv(ifine),file=fn,form='unformatted',status='old')
          write(8,'(a,i2,a,a,/,a,i2)') ' open unit ',ifkv(ifine),
     &         ' file ',fn,' for kv for fine grid ',
     &         ifine
        else
          write(8,'(a,a,i2)') ' no file opened for kv ',
     &         'for fine grid ',ifine
        endif
 4570 continue
c
c-----Optionally open pressure files for any fine grid
c
      do 4575 ifine=1,numfin
        read(7,7001) ifprs(ifine),fn
        if (ifprs(ifine) .gt. 0) then
          open(ifprs(ifine),file=fn,form='unformatted',status='old')
          write(8,'(a,i2,a,a,/,a,i2)') ' open unit ',ifprs(ifine),
     &         ' file ',fn,' for pressure for fine grid ',
     &         ifine
        else
          write(8,'(a,a,i2)') ' No file opened for pressure ',
     &         'for fine grid ',ifine
        endif
 4575 continue
c
c-----Optionally open ozone files for any fine grid
c
      do 4580 ifine=1,numfin
        read(7,7001) ifo3(ifine),fn
        if (ifo3(ifine) .gt. 0) then
          open(ifo3(ifine),file=fn,form='unformatted',status='old')
          write(8,'(a,i2,a,a,/,a,i2)') ' open unit ',ifo3(ifine),
     &         ' file ',fn,' for ozone for fine grid ',
     &         ifine
        else
          write(8,'(a,a,i2)') ' no file opened for ozone ',
     &         'for fine grid ',ifine
        endif
 4580 continue
c
c-----Optionally open and read surface files for any fine grid
c
      do 4590 ifine=1,numfin
        read(7,7001) ifsrf(ifine),fn
        if (ifsrf(ifine) .gt. 0) then
          open(ifsrf(ifine),file=fn,form='unformatted',status='old')
          write(8,'(a,i2,a,a,/,a,i2)') ' open unit ',ifsrf(ifine),
     &         ' file ',fn,' for surface char. for fine grid ',
     &         ifine
          call rdfsrf(nfx(ifine),nfy(ifine),mlus,ifsrf(ifine),
     &                flandf(1,1,ifine))
          close(ifsrf(ifine))
        else
          write(8,'(a,a,i2)') ' no file opened for surface char. ',
     &         'for fine grid ',ifine
        endif
 4590 continue
c
c-----Open emissions files for fine grids
c
      do 4595 ifine=1,numfin
        read(7,7001) ifaem1(ifine),fn
        if (ifaem1(ifine) .gt. 0) then
          open(ifaem1(ifine),file=fn,form='unformatted',status='old')
          write(8,'(a,i2,a,a,/,a,i2)') ' open unit ',ifaem1(ifine),
     &         ' file ',fn,' for weekday emissions for fine grid ',
     &         ifine
        else
          if (larea) then
            write(*,*)'Treatment of area sources has been specified'
            write(*,*)'you must provide an area source emissions file'
            stop
          endif
          write(8,'(a,a,i2)') ' no file opened for weekday emissions ',
     &         'for fine grid ',ifine
        endif
        read(7,7001) ifaem2(ifine),fn
        if (ifaem2(ifine) .gt. 0) then
          open(ifaem2(ifine),file=fn,form='unformatted',status='old')
          write(8,'(a,i2,a,a,/,a,i2)') ' open unit ',ifaem2(ifine),
     &         ' file ',fn,' for saturday emissions for fine grid ',
     &         ifine
        else
          write(8,'(a,a,i2)') ' no file opened for saturday emissions ',
     &         'for fine grid ',ifine
        endif
        read(7,7001) ifaem3(ifine),fn
        if (ifaem3(ifine) .gt. 0) then
          open(ifaem3(ifine),file=fn,form='unformatted',status='old')
          write(8,'(a,i2,a,a,/,a,i2)') ' open unit ',ifaem3(ifine),
     &         ' file ',fn,' for sunday emissions for fine grid ',
     &         ifine
        else
          write(8,'(a,a,i2)') ' no file opened for sunday emissions ',
     &         'for fine grid ',ifine
        endif
 4595 continue
c
c-----Initialize all flux tracking arrays to zero
c
      call flxnit(nospec,numfin)
c
c-----Write instantaneous file headers
c
      tb = tstr/100.
      te = tend/100.
      if (lcart) then
        dx = dlong*1000.
        dy = dlat*1000.
        xu = xorg*1000.
        yu = yorg*1000.
      else
        dx = dlong
        dy = dlat
        xu = wlong
        yu = slat
      endif
c
      write(51) (ifili(i:i),mblank,mblank,mblank,i=1,10),
     &             (msg(i:i),mblank,mblank,mblank,i=1,60),ndum,
     &             nospc1,idystr,tb,idaynf,te
      write(51) fzero,fzero, utmzon, xu, yu, dx, dy,
     &          nox, noy, nlayer, izero, izero, fzero, fzero, fzero
      write (51)  izero,izero,nox,noy
      write (51) ((mspec(m,l),m=1,10),l=1,nospc1)   
c---- write instantaneous file headers for just OH and Ozone when necessary.
c     dfg, 4/14/99
c
      if (lmoh) then
        do k=1,nlayer
          do j=1,noy
            do i=1,nox
              ohcon(i,j,k)=1.0E-12
              ho2con(i,j,k)=1.0E-12
              no3con(i,j,k)=1.0E-12
              aso2con(i,j,k)=1.0E-12
              peccon(i,j,k)=1.0e-12
            end do
          end do
        end do
        write(iohf) (ifili(i:i),mblank,mblank,mblank,i=1,10),
     &             (msg(i:i),mblank,mblank,mblank,i=1,60),ndum,
     &             7,idystr,tb,idaynf,te
        write(iohf) fzero,fzero, utmzon, xu, yu, dx, dy,
     &          nox, noy, nlayer, izero, izero, fzero, fzero, fzero
        write (iohf)  izero,izero,nox,noy
        write (iohf) (msoh(m),m=1,10)
        write (iohf) (msho2(m),m=1,10)
        write (iohf) (msno3(m),m=1,10)
        write (iohf) (msaso2(m),m=1,10)
        write (iohf) (mspec(m,kh2o2),m=1,10)
        write (iohf) (mspec(m,ko3),m=1,10)
        write (iohf) (mspec(m,kpec),m=1,10)
      endif
c
c-----Identify species for avg and deposition output files
c
      if (iuavg .gt. 0) then
        if (navspc .le. 0) then
          navspc = nospec
          do 76 l=1,nospec
            kavspc(l) = l
            write(mavspc(l),'(10a1)') (mspec(i,l),i=1,10)
   76     continue
        else
          do 78 l=1,nospec
            write(nams,'(10a1)') (mspec(i,l),i=1,10)
            do 77 lav=1,navspc
              if (nams .eq. mavspc(lav)) then
                kavspc(lav) = l
              endif
   77       continue
   78     continue
        endif
c
c       Check that all average species have a valid pointer
c
        do 85 l=1,navspc
          if (kavspc(l) .le. 0 .or. kavspc(l) .gt. nospec) then
            write(*,'(a)') '***ERROR IN SETUP ***'
            write(*,'(a,a)') 'Invalid average species: ', mavspc(l)
            stop
          endif
   85   continue
c
c-----Write average and deposition file headers
c
        write(iuavg) (ifile(i:i),mblank,mblank,mblank,i=1,10),
     &               (msg(i:i),mblank,mblank,mblank,i=1,60),ndum,
     &               navspc,idystr,tb,idaynf,te
        write(iuavg) fzero,fzero, utmzon, xu, yu, dx, dy,
     &            nox, noy, ndum, izero, izero, fzero, fzero, fzero
        write (iuavg)  izero,izero,nox,noy
        write (iuavg) ((mavspc(l)(m:m),mblank,mblank,mblank,m=1,10),
     &                l=1,navspc)   
      endif
      if (iudep .gt. 0) then
        write(iudep) (ifild(i:i),mblank,mblank,mblank,i=1,10),
     &               (msg(i:i),mblank,mblank,mblank,i=1,60),ndum,
     &               navspc,idystr,tb,idaynf,te
        write(iudep) fzero,fzero, utmzon, xu, yu, dx, dy,
     &            nox, noy, ndum, izero, izero, fzero, fzero, fzero
        write (iudep)  izero,izero,nox,noy
        write (iudep) ((mavspc(l)(m:m),mblank,mblank,mblank,m=1,10),
     &                l=1,navspc)   
c
c-----Initialize deposition variables to zero
c
        do 86 l = 1,nospec
          do 87 j = 1,noy
            do 88 i = 1,nox
               dposit(i,j,l) = 0.
               wposit(i,j,l) = 0.
   88       continue
   87     continue
   86   continue
        do 89 n = 1,maxfin
          do 90 l = 1,mxspec
            do 91 i = 1,mxfxby
             dpositf(i,l,n) = 0.
             wpositf(i,l,n) = 0.
   91       continue
   90     continue
   89   continue
      endif
c
c-----Check for bad dimensions
c
      noxy=nox*noy
      write(*,3004)
 3004 format(' read all of input file ')
      time=tstr
      ndate=idystr
      totime=0.
c
c-----Read point source emissions data
c
      if (lpts) then
        call hedpts(nospec, nox, noy, edf, nemit, kemit)
        write(*,3013) nopts,edf
 3013   format(' reading point source data nopts=',2i10)
        write(8,1080) nopts, larea
 1080   format(//i10,' point sources are involved in this simulation; ',
     &   ' area source emissions= ',l5,5x)
c
c       Check if suppressing output
c
        do 140 i=1,nopts
          if (lsuprs.and.(i.gt.5)) go   to 140
c
          write(8,150) i
          write(8,152) plon(i),plat(i),ptsrcx(i),ptsrcy(i),stkht(i),
     &                 stkd(i),temps(i),vexit(i)
  150     format(' source number ',i5)
  152     format(' stack parameters   ',/,8f10.2)
  140   continue
c
        write(*,3014)
 3014   format(' finish reading points')
        write(8,226) nopts
        write(*,226) nopts
  226   format(' in setup nopts = ', i10)
      endif
c
c-----If area sources are present, read headers
c
      if (larea) then
        call gunit(aem,time,ndate)
        do 1901 ifine=1,numfin
          if (aem .eq. aem1) then 
            ifaem(ifine) = ifaem1(ifine)
          else if (aem .eq. aem2) then
            ifaem(ifine) = ifaem2(ifine)
          else if (aem .eq. aem3) then
            ifaem(ifine) = ifaem3(ifine)
          endif
 1901   continue
        if (idiag .ne. 0) then
          write(*,3019)time,ndate,aem
 3019     format(' after call gunit ',f10.1,i10,i10)
        endif
        call hedqar(nospec,nox,noy,aem,naemit,kaemit,katol,kaxyl)
        if (idiag .ne. 0) then
          write(*,'(a)') ' back from hedqar'
        endif
        do 1955 ifine=1,numfin
          if (ifaem(ifine) .gt. 0) then
            call hedqar(nospec,nfx(ifine),nfy(ifine),ifaem(ifine),
     &                  nfemit(ifine), kfemit(1,ifine),
     &                  kaftol(ifine), kafxyl(ifine))
            if (idiag .ne. 0) then
              write(*,'(a)') ' back from hedqar'
            endif
          endif
 1955   continue
        kntaem=0
      endif
c
c     If area sources included, skip to correct time
c
      if (larea) then
        call getqar(mxx,mxy,nox,noy,naemit,aem,time,ndate,
     &              emtim1,iemdy1,emtim2,iemdy2,kaemit,
     &              katol,kaxyl,qarea,iqeof)
        if (iqeof .ne. 0) then
          write(*,'(a,i4,a,f10.2,i10)') ' end of file on unit',
     &         aem,' at ',time,ndate
          stop
        endif
        if (idiag .ne. 0) then
          write(*,'(a)') ' back from getqar'
        endif
        do 1902 ifine=1,numfin
          if (ifaem(ifine) .gt. 0) then
            call getqar(nfx(ifine),nfy(ifine),
     &                  nfx(ifine),nfy(ifine),nfemit(ifine),
     &                  ifaem(ifine),time,ndate,
     &                  emtm1f(ifine),iemd1f(ifine),
     &                  emtm2f(ifine),iemd2f(ifine),
     &                  kfemit(1,ifine),kaftol(ifine),kafxyl(ifine),
     &                  qfarea(1,1,ifine),iqeof)
            if (iqeof .ne. 0) then
              write(*,'(a,i4,a,f10.2,i10)') ' end of file on unit',
     &              ifaem(ifine),' at ',time,ndate
              stop
            endif
            if (idiag .ne. 0) then
              write(*,'(a)') ' back from getqar'
            endif
          endif
 1902   continue
      endif
c
      if (lpts) then
        call getpt(time,ndate,edf,nemit,iqeof)
        if (iqeof .ne. 0) then
          write(*,'(a,i4,a,f10.2,i10)') ' end of file on unit',
     &         edf,' at ',time,ndate
          stop
        endif
      endif
c
c-----Add up total area emissions
c
      do 278 i=1,mxspec
        tarea(i,1)=0.0
        tarea(i,2)=0.0
  278 continue
      do 175 ll=1,naemit
        l = kaemit(ll)
        if ( l .le. 0 ) goto 175
        do 177 j=1,noy
          do 173 i=1,nox
            tarea(l,1)=tarea(l,1)+qarea(i,j,ll)
  173     continue
  177   continue
  175 continue
      write (*,*) 'nemit =', nemit, nopts, mxemit, mxsrc, mxspec
      do 178 ll=1,nemit
        l=kemit(ll)
c        write (*,*) ll,l,kemit(ll)
        if ( l .le. 0 ) goto 178
        do 176 i=1,nopts
          tarea(l,2)=tarea(l,2)+ptemis(ll,i)
  176   continue
  178 continue
      write(8,301)
      write(8,300) (tarea(l,1),l=1,nospec)
      write(8,302) (tarea(l,2),l=1,nospec)
  301 format(' total area and point source emissions (moles/min)')
  300 format(' area ',8(1x,e12.4))
  302 format('point ',8(1x,e12.4))
c
c-----Total area source emissions on fine grids
c
      do 98 ifine=1,numfin
c
c***Passing mxemit instead of nospec because the modeled species
c     (nospec) may be smaller than emitted species.  dfg, 1999/04/16
c   passing two parameters now
c
        call eftot(nfx(ifine),nfy(ifine),nfemit(ifine),
     &                  qfarea(1,1,ifine),ifine,idebug)
c       call eftot(nfx(ifine),nfy(ifine),nospec,
c    &                  qfarea(1,1,ifine),ifine,idebug)
 98      continue
c
c-----Calculate grid spacing in km
c
      write(*,3021)
 3021 format(' call grdsp')
      call grdsp
c
c  *** set initial radical concentration is 1.e-7
c
      do j=1,nox*noy*nlayer
        do i=1,mxrads
          radcon(i,j)=1.0e-12
          if (i .eq. 10) then
            radcon(i,j) = 1.0e-16
          end if
        end do
      end do
c
c-----Read initial concentrations in ppm
c
      if (.not.lrstrt) then
        call hedibc(nospec,nox,noy,icbc,nicsp,kicsp,mdum)
        if (mdum .ne. ifili) liccon = .true.
        call getic(time,ndate,icbc,iceof)
        if (iceof .ne. 0) then
          write(*,*) ' end of file on ic file'
          stop
        endif
      endif
      write (*,*) 'end of hedibc'
      if (icbc .gt. 0) then
        close (icbc)
        write(8,'(a,i3,a)') ' closed ic file (unit ',icbc,')'
      endif
c
c*** call hedo3 for o3 oh no3 h2o2 ho2 file.
c
      if (lo3) call hedo3(time,date)
c
c-----Read boundary file unit
c
      read(7,7001) icbc,fn2
c
c-----Open deposition velocity file
c
      read(7,7001) idepv,fn
      if ( idepv .gt. 0 ) then
         open(idepv,file=fn,form='unformatted',status='old')
         write(8,7028) idepv,fn
	 ldepn=.false.
         call heddv(nospec)
      else
         if (ldepn) then
           write(8,7029)
         else
           write(8,7040)
         endif
      endif
 7028 format(' open unit=',i2,'= file=',a,'for deposition velocities',
     &        /,'  ldepn will be set to .false.')
 7029 format(' no file opened for deposition velocities...',/,
     &    ' dry deposition will be calculated internally.')
 7040 format(' no file opened for deposition velocities...',/,
     &    ' no dry deposition will be calculated.')
c
c-----Before we get too far, check unit numbers for conflicts
c
      call unchek
c
c-----Read the boundary condition headers; concentrations are read in update
c
      open(icbc,file=fn2,form='unformatted',status='old')
      write(8,7027) icbc,fn2
 7027 format(' open unit=',i2,'  file=',a,'for boundary concentrations')
      write (*,*) 'start of hedibc and bnddef'
      call hedibc(nospec,nox,noy,icbc,nbcsp,kbcsp,mdum)
      call bnddef(icbc,nox,noy,istr,iend,jstr,jend)
      write (*,*) 'end of hedibc and bnddef'
c
c-----Write out boundary definition to unit 8
c
      write(8,2080) noy
      write(8,2081) (istr(j),iend(j),j=1,noy)
      write(8,2082) nox
      write(8,2081) (jstr(i),jend(i),i=1,nox)
 2080 format(/,' Boundary definition for ',i3,' rows')
 2082 format(/,' Boundary definition for ',i3,' columns')
 2081 format(10('(',i3,',',i3,')'))
      write(8,*)
c
c-----If restarting read old concentrations
c
      if (lrstrt) then
         write (*,*) 'before restrt', time, ndate
        call restrt(time,ndate)
      endif
c
      write (*,*) 'enter diagn'
      call diagn(time,ndate,1,tend,tstr,idystr
     &,idaynf)
c 
c-----Update reads the meteorological data
c
      write (*,*) 'enter update'
      call update(time,ndate,tdatnx,totime,idebug)
c
      write(*,*) 'Returned from update'
      write(*,*) 'idiag', idiag
cdiag
c-----Variables lsnow and lstres are included for future development
c     for now, we just initialize them to zeros
c
      write (*,*) 'before 206'
      do 206 j=1,noy
        do 205 i=1,nox
          lsnow(i,j) = 0
          lstres(i,j) = 0
  205   continue
  206 continue
      write (*,*) 'after 206'
      do 207 j=1,mxfxby
        lsnowf(j) = 0
        lstrsf(j) = 0
  207 continue
      write (*,*) 'after 207'
c
      if (idiag .ne. 0) then
        write(*,3015)
 3015   format(' leave setup ',f10.1,i10)
      endif
      write (*,*) 'end of setup'
      return
      end
