c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine smolar (dt,deltax)
c
c-----ATDM v1.00
c
c     Performs advection using scheme of Smolarkiewicz (1983)
c
c-----95/02/22:
c     -from UAM-V Revision 1.2  94/02/10  21:00:27  tcm
c     -moved adjustment to concentration vector due to wet deposition
c      to STEPX and STEPXF
c-----95/09/12:
c     -made revision to wq at boundary (follows UAM-V change made on
c      95/08/25 by TCM)
c
c-----96/11/06:
c     -Removed commented code for diffusion (reactivated diffusion)
c
c     Input arguments:
c               dt:             time step (min)    
c               deltax:         grid size (km)
c     Called by:
c               stepxy    stepxyf 
c
      include 'parm.cmd'
      include 'advec.cmd'
c
      common /shas/ w(mxxx),c(mxxx),cb(mxxx),
     &              jadf(mxxx),nm1,b1,b2
c
      dimension a(mxxpl2)
      dimension wm(mxxx),wmd(mxxx)
c
c     check beginning boundary
c     -if wind is blowing out, redefine boundary concentration
c     -otherwise use inflow boundary conditions
c
      if (lstagr) then
        wchk = w(1)
      else
        wchk = 0.5*(w(1) + w(2))
      endif
      if (wchk.lt.0.) then
         c(1) = c(2)
      else
         c(1) = b1
      endif
      a(1) = c(1)
c
c     check ending boundary
c     -if wind is blowing out, redefine boundary concentration
c     -otherwise use inflow boundary conditions
c
      n = nm1+1
      if (lstagr) then
        wchk = w(nm1)
      else
        wchk = 0.5*(w(nm1) + w(n))
      endif
      if (wchk.gt.0.) then
         c(n) = c(nm1)
      else
         c(n) = b2
      endif
      a(n) = c(n)
c
c  corrected below -- factor of ca does not belong in wq until
c  after wmq has been calculated.  (tcm, 8/25/95)
c
      ca = dt/deltax
      if (lstagr) then
c       wq = w(1)*ca
        wq = w(1)
      else
c       wq = 0.5*(w(1)+w(2))*ca
        wq = 0.5*(w(1)+w(2))
      endif
      wmq = (abs(wq)*deltax - wq*wq*dt)/deltax
      wq = wq*ca
      f2 = 0.5*((wq + abs(wq))*c(1) + (wq - abs(wq))*c(2))
      do 90 i=2,nm1
        if (lstagr) then
          wm(i) = w(i)
        else
          wm(i) = 0.5*(w(i) + w(i+1))
        endif
        wmd(i) = (abs(wm(i))*deltax - wm(i)*wm(i)*dt)/deltax
   90 continue
c
c     advection loop
c
      do 100 i = 2,nm1
        f1 = f2
        wp = wm(i)*ca
        abswp = abs(wp)
        f2 = 0.5*((wp + abswp)*c(i) + (wp - abswp)*c(i+1))
        a(i) = c(i) - (f2 - f1)
  100 continue
c
      do 299 kk = 1,2
        wadq = wmq*(a(2) - a(1))/(a(1) + a(2) + 1.0e-15)
        wmq = (abs(wadq)*deltax - dt*wadq*wadq)/deltax
        abswdq = abs(wadq)
        f2 = 0.5*ca*((wadq + abs(wadq))*a(1) + (wadq - abs(wadq))*a(2))
c
c     don't do anti-diffusion adjustment on boundary
c
        f2 = 0.
c
c     calculate anti-diffusion velocity
c
        do 200 i = 2,nm1
          f1 = f2
          wadm = wmd(i)*(a(i+1) - a(i))/(a(i) + a(i+1) + 1.0e-15)
          wmd(i) = (abs(wadm)*deltax - dt*wadm*wadm)/deltax
          f2 = 0.5*ca*((wadm + abs(wadm))*a(i) + 
     &                 (wadm - abs(wadm))*a(i+1))
c
c     don't do anti-diffusion adjustment on boundary or on nested grid
c     interfaces
c
          if (i.eq.nm1) f2 = 0.
          if (jadf(i).ne.0) f2 = 0.
          a(i) = a(i) - (f2 - f1)
  200   continue
  299 continue
c 
c     horizontal diffusion--cb(i)=kh(i)*deltat/(deltax)**2
c
      do 301 i=2,nm1
c         c(i) = a(i)
        if (i.eq.2 .or. jadf(i-1).ne.0) then
          c(i) = a(i) + cb(i)*(a(i+1) - a(i))
        elseif (i.eq.nm1 .or. jadf(i).ne.0) then
          c(i) = a(i) - cb(i-1)*(a(i) - a(i-1))
        else
          c(i) = a(i) + cb(i)*(a(i+1) - a(i)) - cb(i-1)*(a(i) - a(i-1))
        endif
  301 continue
 9999 return
      end
