c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine toxpart(pm10,temp,toxg,toxp,n)
c
c-----ATDM v1.00
c
c     Partitions POM1, POM2, DIOXIN and ATRAZ into gas and particulate
c     fractions
c
c-----95/07/18:
c     -from UAM6.40tox, 3/31/92
c     -Modified for ATDM toxic species
c-----95/08/07:
c     -Revised to operate on one species per call
c     -revisions to ensure consistent use of temperature dependence in PL
c-----95/10/05:
c     -pm10 now coming in as ug/m3
c
c     Input arguments:
c               pm10:           Concentration of total PM10 (ug/m3)
c               temp:           Ambient temperature (K)
c               toxg:           Gas-phase concentration (ppm): 
c               toxp:           Particle-phase concentration (ppm): 
c               n:              Species index in the following order:
c                               1=POM1, 2=POM2, 3=DIOXIN, 4=ATRAZ
c     Output arguments:
c               toxg:           Gas-phase concentration (ppm): 
c               toxp:           Particle-phase concentration (ppm): 
c     Called by:
c               rchem     rchemf    vrtdif    fvrtdif
c
      dimension pl0(4), tdep(4)
      data pl0 /1.4e-4, 5.0e-7, 9.0e-7, 8.0e-6/
      data tdep /6600., 7600., 7000., 7000./
c
      tox = toxp + toxg
      tfac = 1./298. - 1./temp
      pl = pl0(n)*exp(tdep(n)*tfac)
      fp = 1./(3.55e+7*pl/pm10 + 1.)
      toxp = fp*tox
      toxg = tox - toxp
c
      return
      end
