c
c     Copyright 1996, 2001 Systems Applications International
c       The REMSAD modeling system software is the property of SAI
c       and is protected by registered copyrights. Distribution of
c       the software outside of your organization is not permitted.
c       Preparation and distribution of any derivative software using
c       any portions of the REMSAD code, without the express permission
c       of SAI, is also not permitted. By accepting the REMSAD code
c       provided in this release, you are also accepting the
c       limitations, terms, and provisions of use as noted above.
      subroutine vrtdif(deltat,idebug)
c
c-----ATDM v1.00
c
c     Performs explicit vertical diffusion calculation on the coarse grid
c     in sigma coordinates
c
c-----95/06/14:
c     C Emery SAI
c-----95/06/23:
c     -revised mfac and vfac for use with sigma rather than height
c     -added data statements for amolwt
c-----95/08/08:
c     -added nitrate equilibrium calculation and toxics partitioning
c-----95/10/04:
c     -added special case for dtdep > dtdif in calculation of diffusion
c      timestep
c-----95/10/05:
c     -revised calculation of PM10 for toxic partitioning to be in units
c      of ug/m3 rather than ppm
c-----95/11/08:
c     -added code to skip cells covered by a fine grid
c-----95/12/11:
c     -added declarations for nh3,nh4n,nh4s (REAL)
c-----97/02/21:
c     -passed k to subroutine nitrat now. (now replaced by remeql3)
c-----97/06/25:DFG
c     -passed pressure (i.e. pecll) to nitrat subroutine.(now replaced by remeql3)
c
c     Based on routines from SAQM/RADM and UAM-V v1.23 Revision 1.2 94/02/10
c
c     Input arguments:
c               deltat:         time step (min)
c               idebug:         diagnostic debug flag 
c     Routines called:
c               remeql3    toxpart
c     Called by:
c               atdm
c
      include 'parm.cmd'
      include 'parchm.cmd'
      include 'parfin.cmd'
      include 'balanc.cmd'
      include 'comuam.cmd'
      include 'com3.cmd'
      include 'comchem.cmd'
      include 'ctlflg.cmd'
      include 'deffin.cmd'
c
      real sigmah(mxz),dsigh(mxz),dsigi(mxz),wt(mxz)
      real kfact(mxz), mfac
      real nh3,nh4s,nh4n
      logical lprnt
      data eps /0.622/, rdry /287./, grav /9.8/, amolwt /28.8/
      data e0 /6.11/, lv /2.5e6/, rv /461./
c
c-----Bit 8 (128's location) turns on debug writes in vrtdif
c
      idiag = and(idebug,128)
      if (idiag.ne.0) write(*,*)'Entering VRTDIF, deltat,lpmchm = ',
     &                                      deltat,lpmchm
c
c-----Calculate depth parameters
c
      kmax = nlayer
      do 10 k = 1,kmax
        sigmah(k) = (sigma(k) + sigma(k+1))/2.
        dsigh(k) = sigma(k+1) - sigma(k)
   10 continue
      do 11 k = 1,kmax-1
        dsigi(k) = 1./(sigmah(k+1) - sigmah(k))
   11 continue
      if (idiag.ne.0) then
        write(*,*)'sigmah,dsigh,dsigi:'
        write(*,*) (sigmah(k),k=1,kmax)
        write(*,*) (dsigh(k),k=1,kmax)
        write(*,*) (dsigi(k),k=1,kmax-1)
      endif
c
c-----Loop over rows and columns
c
c      write (*,*) 'before 100'
      do 100 j = 1,noy
c
c-----Check if outside of modeling domain
c
        if (istr(j).gt.nox) go to 100
        i1 = istr(j) + 1
        i2 = iend(j) - 1
        do 110 i = i1,i2
c           write (*,*) 'i j', i,j
          if (j.le.jstr(i)) goto 110
          if (j.ge.jend(i)) goto 110
c
c-----Skip cells covered by fine grid
c
          do 115 ifine = 1,numfin
            if (i.ge.ixfb(ifine) .and. i.le.ixfe(ifine) .and.
     &          j.ge.jyfb(ifine) .and. j.le.jyfe(ifine)) goto 110
  115     continue
c
          lprnt = .false.
          if (idiag.ne.0.and.i.eq.nox/2.and.j.eq.noy/2) lprnt = .true.
          do 120 k = 1,kmax-1
c
c-----Determine nitrate equilibrium if this is a PM chemistry run, or if
c     its a Toxic chemistry run with nitrates
c
            pcell = press(i,j,k)
            tcell = temper(i,j,k)
c           if (lpmchm .or. (ltxchm .and. khno3.ne.0)) then
c             ev = ch2o(i,j,k)*press(i,j,k)/(ch2o(i,j,k) + eps)
c             es = e0*exp((lv/rv)*(1./273. - 1./tcell))
c             rhcell = 100.*ev/es
c
c               aso4 and gso4 should not be modified by remeql3
c               so calculation of fractions and redistribution
c               of sulfate has been removed.
c               since prior versions of REMSAD have assumed
c               that all pariculate nitrate is nh4no3,
c               we will assume that here also.  This means
c               that we do not need to calculate the fraction
c               of nh4 as nitrate (frnh4n) before remeql3.
c               The assumption that this fraction would be
c               the same after remeql3 is 
c                 tso4  = conc(i,j,kaso4,k) + conc(i,j,kgso4,k)
c                 thno3 = conc(i,j,khno3,k)
c                 tnh3o = conc(i,j,knh3,k)
c                 pno3  = conc(i,j,kpno3,k)
c                 panh4 = conc(i,j,knh4n,k) + conc(i,j,knh4s,k)
Ctcm              frnh4n= conc(i,j,knh4n,k)/panh4
Ctcm              paso4 = conc(i,j,kaso4,k) + conc(i,j,kgso4,k)
Ctcm              fraso4= conc(i,j,kaso4,k)/paso4
c                  write (*,*) 'before rem',i,j,k
c                 call remeql3(tso4,thno3,tnh3o,rhcell,tcell,
c    &                 pcell,pno3,panh4,paso4,i,j,k)
c                  write (*,*) 'after rem',i,j,k
c
c     check against lower bound for these species
c
c                 conc(i,j,khno3,k) = amax1(thno3,bdnl(khno3))
c                 conc(i,j,knh3,k)  = amax1(tnh3o,bdnl(knh3))
Ctcm              conc(i,j,kaso4,k) = amax1(paso4*fraso4,bdnl(kaso4))
Ctcm              conc(i,j,kgso4,k) = amax1(paso4*(1-fraso4),bdnl(kgso4))
c                 conc(i,j,kpno3,k) = amax1(pno3,bdnl(kpno3))
Ctcm              conc(i,j,knh4n,k) = amax1(panh4*frnh4n,bdnl(knh4n))
Ctcm              conc(i,j,knh4s,k) = amax1(panh4*(1-frnh4n),bdnl(knh4s))
c                 conc(i,j,knh4n,k) = amax1(pno3,bdnl(knh4n))
c                 conc(i,j,knh4s,k) = amax1(panh4-pno3,bdnl(knh4s))
c           endif
c
c-----Partition certain toxic species between vapor and particle phase
c
            if (ltxchm) then
              pm10 = 4.
              mfac = cpfac*(1013.25/press(i,j,k))*(tcell/273.)
              do 35 l = ngas+1,nospec
                if (l.eq.kpom1p .or. l.eq.kpom2p .or. l.eq.kdioxp .or.
     &            l.eq.katrazp) goto 35
                lsp = jind(l)
                pm10 = pm10 + molwt(lsp)*conc(i,j,l,k)/mfac
  35          continue
              if (kpom1.ne.0)
     &          call toxpart(pm10,tcell,conc(i,j,kpom1,k),
     &                       conc(i,j,kpom1p,k),1)
              if (kpom2.ne.0) 
     &          call toxpart(pm10,tcell,conc(i,j,kpom2,k),
     &                       conc(i,j,kpom2p,k),2)
              if (kdiox.ne.0)
     &          call toxpart(pm10,tcell,conc(i,j,kdiox,k),
     &                       conc(i,j,kdioxp,k),3)
              if (katraz.ne.0)
     &          call toxpart(pm10,tcell,conc(i,j,katraz,k),
     &                       conc(i,j,katrazp,k),4)
            endif
c
c            write (*,*) 'before kv factor'
c-----Calculate density and Kv-factor profile
c
            pbar = 100.*(press(i,j,k)+press(i,j,k+1))/2.
            tvk = temper(i,j,k)*(eps + ch2o(i,j,k))/
     &            (eps*(1. + ch2o(i,j,k)))
            tvkp1 = temper(i,j,k+1)*(eps + ch2o(i,j,k+1))/
     &              (eps*(1. + ch2o(i,j,k+1)))
            tvbar = (tvk + tvkp1)/2. 
            densf = pbar/(rdry*tvbar)
            kfact(k) = kv(i,j,k)*densf**2*60.*deltat*dsigi(k)*
     &                 (grav/(100.*pstar(i,j)))**2
c           if (lprnt .and. idiag.ne.0) then
c             write(*,*)'k,kv,pbar,tvbar,densf,pstar,kfact'
c             write(*,*) k,kv(i,j,k),pbar,tvbar,densf,pstar(i,j),
c    &                   kfact(k)
c           endif
 120      continue
c          write (*,*) 'after 120'
          vfac = -1.e6*deltax(j)*deltay*dsigh(1)
          mfac = pstar(i,j)/(10.*grav*amolwt)
c
c-----Determine maximum time step and number of sub-time steps
c
          dtmin = 60.*deltat
          do 130 k = 1,kmax-1
            dt = 60.*deltat*dsigi(k)*dsigh(k)**2/(2.*kfact(k))
            dtmin = amin1(dtmin,dt)
c           if (lprnt .and. idiag.ne.0) write(*,*)'k,deltat,dt,dtmin',
c    &                                             k,deltat,dt,dtmin
 130      continue
          if (dtmin.ge.60.*deltat) then
            ndt = 1
          else
            ndt = int(60.*deltat/dtmin) + 1
          endif
          dtdif = 60.*deltat/float(ndt)
c         if (lprnt .and. idiag.ne.0) write(*,*)'ndt,dtdif',ndt,dtdif
c
c-----Loop over all species
c
          do 140 l = 1,nospec
c             write (*,*) '140=', l
            lsp = jind(l)
c
c-----Load deposition rate
c     calculate deposition velocity of NH4S as a weighted average of
c     dep velocity for GSO4 and ASO4
c
c     calculate step size for deposition (make it one quarter
c     of max possible)
c
            if (l.eq.knh4s) then
               tconc = conc(i,j,kgso4,1) + conc(i,j,kaso4,1)
               dep = -(depvel(i,j,kgso4)*conc(i,j,kgso4,1) +
     &                 depvel(i,j,kaso4)*conc(i,j,kaso4,1))/tconc
            else
               dep = -depvel(i,j,l)
            endif
            dtdep = h(i,j,1)/(4.*amax1(dep,1.e-6))
            dt = amin1(dtdif,dtdep)
            if (dt.eq.dtdif) goto 145
            if (dt.ge.60.*deltat) then
              ndt = 1
            else
              ndt = int(60.*deltat/dt) + 1
            endif
 145        dt = 60.*deltat/float(ndt)
c           if (lprnt .and. idiag.ne.0) write(*,*)'l,dtdep,dt,ndt',
c    &                                             l,dtdep,dt,ndt
c
c-----Start of vertical diffusion; loop over sub time-steps
c
            do 150 n = 1,ndt
c
c   dpflow and delc1 in moles; h in m; dposit in g/km2
c
            delc1 = (dt*dep/h(i,j,1))*conc(i,j,l,1)*vfac*mfac
            dpflow(l) = dpflow(l) - delc1
            dposit(i,j,l) = dposit(i,j,l) + molwt(lsp)*delc1/
     &                      (deltay*deltax(j))
c           if (lprnt .and. idiag.ne.0) then
c             write(*,*)'VRTDIF: l,n',l,n
c             write(*,*)'dt,dep,h(i,j,1),conc(i,j,l,1),vfac,mfac'
c             write(*,*) dt,dep,h(i,j,1),conc(i,j,l,1),vfac,mfac 
c             write(*,*)'delc1,molwt,deltay,deltax'
c             write(*,*) delc1,molwt(lsp),deltay,deltax(j)
c             write(*,*)'dposit',dposit(i,j,l)
c           endif
c
            wt(1) = conc(i,j,l,1)
     &            + kfact(1)*(conc(i,j,l,2) - conc(i,j,l,1))
     &              /(dsigh(1)*float(ndt))
     &            - conc(i,j,l,1)*dt*dep/h(i,j,1)
            wt(1) = amax1(wt(1),0.)
c
            do 160 k = 2,kmax-1
              wt(k) = conc(i,j,l,k)
     &              + (kfact(k)*(conc(i,j,l,k+1) - conc(i,j,l,k))
     &              -  kfact(k-1)*(conc(i,j,l,k) - conc(i,j,l,k-1)))
     &                /(dsigh(k)*float(ndt))
              wt(k) = amax1(wt(k),0.)
 160        continue
c
            k = kmax
            wt(k) = conc(i,j,l,k)
     &            - kfact(k-1)*(conc(i,j,l,k) - conc(i,j,l,k-1))
     &              /(dsigh(k)*float(ndt))
            wt(k) = amax1(wt(k),0.)
c
c-----Load concentration array
c
            do 170 k = 1,kmax
c             if (lprnt .and. idiag.ne.0) then
c               write(*,*)'k,conc,wt',k,conc(i,j,l,k),wt(k)
c             endif
              conc(i,j,l,k) = amax1(wt(k),bdnl(l))
c             if (lprnt .and. idiag.ne.0) 
c    &           write(*,*) 'conc',conc(i,j,l,k)
 170        continue
c
c-----End sub-time loop
c
 150        continue
c
 140      continue
 110    continue
 100  continue
c      write (*,*) 'stopped in vrtdif'
c      stop
      return
      end
